/*
** open.c - Function to open a device according to index or serial number.
**
** (C) Copyright 2010 Alpha Data
*/

#include <admxrc3.h>
#include "sdk_common.h"

ADMXRC3_STATUS
sampleOpenCard(
  unsigned int index,
  bool_t bByIndex,
  unsigned int serial,
  bool_t bBySerial,
  bool_t bReadOnly,
  ADMXRC3_HANDLE* phCard)
{
  ADMXRC3_STATUS status;
  ADMXRC3_HANDLE hCard;
  ADMXRC3_CARD_INFO cardInfo;
  unsigned int i;

  if (bByIndex && bBySerial) {
    /* Attempt to open card with specified index and serial number */
    if (bReadOnly) {
      status = ADMXRC3_OpenEx(index, TRUE, 0, &hCard);
    } else {
      status = ADMXRC3_Open(index, &hCard);
    }
    if (status != ADMXRC3_SUCCESS) {
      return status;
    }
    status = ADMXRC3_GetCardInfo(hCard, &cardInfo);
    if (status != ADMXRC3_SUCCESS) {
      return status;
    }
    if (cardInfo.SerialNumber == serial) {
      *phCard = hCard;
      return ADMXRC3_SUCCESS;
    }
    return ADMXRC3_DEVICE_NOT_FOUND;
  }

  if (bBySerial) {
    /* Attempt to open card with specified serial number */
    for (i = 0; i < 256; i++) {
      if (bReadOnly) {
        status = ADMXRC3_OpenEx(i, TRUE, 0, &hCard);
      } else {
        status = ADMXRC3_Open(i, &hCard);
      }
      if (status != ADMXRC3_SUCCESS) {
        if (status == ADMXRC3_DEVICE_NOT_FOUND) {
          return status;
        }
        continue;
      }
      status = ADMXRC3_GetCardInfo(hCard, &cardInfo);
      if (status != ADMXRC3_SUCCESS) {
        continue;
      }
      if (cardInfo.SerialNumber == serial) {
        *phCard = hCard;
        return ADMXRC3_SUCCESS;
      }
      ADMXRC3_Close(hCard);
    }
    return ADMXRC3_DEVICE_NOT_FOUND;
  } else {
    /* Attempt to open card with specified index */
    if (bReadOnly) {
      return ADMXRC3_OpenEx(index, TRUE, 0, phCard);
    } else {
      return ADMXRC3_Open(index, phCard);
    }
  }
}
