#include "stdafx.h"
#include "afxwinappex.h"

#ifdef _DEBUG
# define new DEBUG_NEW
#endif

#include <platform.h>
#include <objmodel.h>

#include "control_id.h"
#include "deviceinfownd.h"

#define ARRAY_LENGTH(x) (sizeof(x) / sizeof((x)[0]))

IMPLEMENT_DYNAMIC(CDeviceInfoWnd, CWnd)

BEGIN_MESSAGE_MAP(CDeviceInfoWnd, CWnd)
    ON_WM_CREATE()
    ON_WM_CLOSE()
    ON_WM_SIZE()
END_MESSAGE_MAP()

// -------------------------------------------------------------------------
// Start of IDeviceInfoWnd interface
// -------------------------------------------------------------------------

void
CDeviceInfoWnd::deviceChanged()
{
#if !defined(HWSIM)
  ADMXRC3_BANK_INFO bankInfo;
#endif
  ADMXRC3_CARD_INFOEX cardInfoEx;
#if !defined(HWSIM)
  ADMXRC3_FLASH_INFO flashInfo;
#endif
  ADMXRC3_FPGA_INFO fpgaInfo;
#if !defined(HWSIM)
  ADMXRC3_MODULE_INFO moduleInfo;
#endif
  ADMXRC3_SENSOR_INFO sensorInfo;
  ADMXRC3_VERSION_INFO versionInfo;
  ADMXRC3_WINDOW_INFO windowInfo;
  ADMXRC3_STATUS status;
  CString str;

  m_treeCtl.SetRedraw(FALSE);

  const DeviceInfo& devInfo = m_pParent->getDeviceInfo();
  if (devInfo.m_hDevice != ADMXRC3_HANDLE_INVALID_VALUE) {
    while (1) {
      HTREEITEM ti = m_treeCtl.GetChildItem(m_treeItem.m_hTargetFpga);
      if (NULL == ti) {
        break;
      }
      m_treeCtl.DeleteItem(ti);
    }
    while (1) {
      HTREEITEM ti = m_treeCtl.GetChildItem(m_treeItem.m_hSensor);
      if (NULL == ti) {
        break;
      }
      m_treeCtl.DeleteItem(ti);
    }
    while (1) {
      HTREEITEM ti = m_treeCtl.GetChildItem(m_treeItem.m_hWindow);
      if (NULL == ti) {
        break;
      }
      m_treeCtl.DeleteItem(ti);
    }
    while (1) {
      HTREEITEM ti = m_treeCtl.GetChildItem(m_treeItem.m_hFlash);
      if (NULL == ti) {
        break;
      }
      m_treeCtl.DeleteItem(ti);
    }
  }

  if (ADMXRC3_HANDLE_INVALID_VALUE != devInfo.m_hDevice) {
    // API version information subtree
    status = ADMXRC3_GetVersionInfo(devInfo.m_hDevice, &versionInfo);
    if (ADMXRC3_SUCCESS == status) {
      str.Format(_T("API libary version: %u.%u.%u"), (unsigned int)versionInfo.Api.Major, (unsigned int)versionInfo.Api.Minor, (unsigned int)versionInfo.Api.Bugfix);
      setTree(m_treeItem.m_hApiItem[0], str);
      str.Format(_T("Driver version: %u.%u.%u"), (unsigned int)versionInfo.Driver.Major, (unsigned int)versionInfo.Driver.Minor, (unsigned int)versionInfo.Driver.Bugfix);
      setTree(m_treeItem.m_hApiItem[1], str);
    } else {
      str.Format(_T("API libary version: (Unable to determine: %s)"), ADMXRC3_GetStatusString(status, TRUE));
      setTree(m_treeItem.m_hApiItem[0], str);
      str.Format(_T("Driver version: (Unable to determine: %s)"), ADMXRC3_GetStatusString(status, TRUE));
      setTree(m_treeItem.m_hApiItem[1], str);
    }

    // Summary information subtree
    status = ADMXRC3_GetCardInfoEx(devInfo.m_hDevice, &cardInfoEx);
    if (ADMXRC3_SUCCESS == status) {
      const TCHAR* pTmp;

      pTmp = CAdmxrc3Help::getModelName(cardInfoEx.Model);
      str.Format(_T("Model: %lu (0x%lX) => %s"),
        (unsigned long)cardInfoEx.Model, (unsigned long)cardInfoEx.Model,
        (NULL != pTmp) ? pTmp : _T("unknown"));
      setTree(m_treeItem.m_hSummaryItem[0], str);
      str.Format(_T("Serial number: %lu (0x%lX)"), (unsigned long)cardInfoEx.SerialNumber, (unsigned long)cardInfoEx.SerialNumber);
      setTree(m_treeItem.m_hSummaryItem[1], str);
      str.Format(_T("Number of target FPGAs: %lu"), (unsigned long)cardInfoEx.NumTargetFpga);
      setTree(m_treeItem.m_hSummaryItem[2], str);
      str.Format(_T("Number of programmable clocks: %lu"), (unsigned long)cardInfoEx.NumClockGen);
      setTree(m_treeItem.m_hSummaryItem[3], str);
      str.Format(_T("Number of DMA channels: %lu"), (unsigned long)cardInfoEx.NumDmaChannel);
      setTree(m_treeItem.m_hSummaryItem[4], str);
      str.Format(_T("Number of windows: %lu"), (unsigned long)cardInfoEx.NumWindow);
      setTree(m_treeItem.m_hSummaryItem[5], str);
      str.Format(_T("Number of sensors: %lu"), (unsigned long)cardInfoEx.NumSensor);
      setTree(m_treeItem.m_hSummaryItem[6], str);
      str.Format(_T("Number of I/O module sites: %lu"), (unsigned long)cardInfoEx.NumModuleSite);
      setTree(m_treeItem.m_hSummaryItem[7], str);
      str.Format(_T("Number of flash memory banks: %lu"), (unsigned long)cardInfoEx.NumFlashBank);
      setTree(m_treeItem.m_hSummaryItem[8], str);
      str.Format(_T("Number of memory banks: %lu"), (unsigned long)cardInfoEx.NumMemoryBank);
      setTree(m_treeItem.m_hSummaryItem[9], str);
      str.Format(_T("Memory bank presence bitmap: 0x%lX"), (unsigned long)cardInfoEx.MemoryBankPresent);
      setTree(m_treeItem.m_hSummaryItem[10], str);

      // Target FPGA subtree
      for (unsigned int i = 0; i < cardInfoEx.NumTargetFpga && i < m_maxNumTargetFpga; i++) {
        status = ADMXRC3_GetFpgaInfo(devInfo.m_hDevice, i, &fpgaInfo);
        if (status == ADMXRC3_SUCCESS) {
          if (fpgaInfo.Present) {
            const TCHAR* pTmp;
            CString identifier = CString(fpgaInfo.Identifier);

            identifier.MakeUpper();
            str.Format(_T("%u  -  %s"), i, identifier);
            if (fpgaInfo.Flags & ADMXRC3_FPGA_SPEEDVALID) {
              str.AppendFormat(_T("-%s"), fpgaInfo.SpeedGrade);
            }
            if (fpgaInfo.Flags & ADMXRC3_FPGA_STEPPINGVALID) {
              str.AppendFormat(_T(" stepping %s"), fpgaInfo.Stepping);
            }
            m_treeItem.m_targetFpgaItem[i].m_hFpga = m_treeCtl.InsertItem(str, m_treeItem.m_hTargetFpga);
            str.Format(_T("Identifier: %s"), (const TCHAR*)fpgaInfo.Identifier);
            m_treeItem.m_targetFpgaItem[i].m_hIdentifier = m_treeCtl.InsertItem(str, m_treeItem.m_targetFpgaItem[i].m_hFpga);
            str.Format(_T("Device code: %lu (0x%lX)"), (unsigned long)fpgaInfo.DeviceCode, (unsigned long)fpgaInfo.DeviceCode);
            m_treeItem.m_targetFpgaItem[i].m_hDeviceCode = m_treeCtl.InsertItem(str, m_treeItem.m_targetFpgaItem[i].m_hFpga);
            pTmp = CAdmxrc3Help::getFamilyString(fpgaInfo.FamilyCode);
            str.Format(_T("Family code: %lu (0x%lX) => %s"),
              (unsigned long)fpgaInfo.FamilyCode, (unsigned long)fpgaInfo.FamilyCode,
              (NULL != pTmp) ? pTmp : _T("unknown"));
            m_treeItem.m_targetFpgaItem[i].m_hFamilyCode = m_treeCtl.InsertItem(str, m_treeItem.m_targetFpgaItem[i].m_hFpga);
            pTmp = CAdmxrc3Help::getSubfamilyString(fpgaInfo.SubfamilyCode);
            str.Format(_T("Subfamily code: %lu (0x%lX) => %s"),
              (unsigned long)fpgaInfo.SubfamilyCode, (unsigned long)fpgaInfo.SubfamilyCode,
              (NULL != pTmp) ? pTmp : _T("unknown"));
            m_treeItem.m_targetFpgaItem[i].m_hFamilyCode = m_treeCtl.InsertItem(str, m_treeItem.m_targetFpgaItem[i].m_hFpga);
            str.Format(_T("Package code: %lu (0x%lX) => %c%c%u"),
              (unsigned long)fpgaInfo.PackageCode, (unsigned long)fpgaInfo.PackageCode,
              (TCHAR)ADMXRC3_PACKAGE_GETTYPE0(fpgaInfo.PackageCode), (TCHAR)ADMXRC3_PACKAGE_GETTYPE1(fpgaInfo.PackageCode), (unsigned int)ADMXRC3_PACKAGE_GETPINS(fpgaInfo.PackageCode));
            m_treeItem.m_targetFpgaItem[i].m_hPackageCode = m_treeCtl.InsertItem(str, m_treeItem.m_targetFpgaItem[i].m_hFpga);
            str.Format(_T("Device: %s"), (unsigned long)fpgaInfo.Device);
            m_treeItem.m_targetFpgaItem[i].m_hDevice = m_treeCtl.InsertItem(str, m_treeItem.m_targetFpgaItem[i].m_hFpga);
            str.Format(_T("Package: %s"), (unsigned long)fpgaInfo.Package);
            m_treeItem.m_targetFpgaItem[i].m_hPackage = m_treeCtl.InsertItem(str, m_treeItem.m_targetFpgaItem[i].m_hFpga);
            str.Format(_T("Flags: 0x%lX"),
              (unsigned long)fpgaInfo.Flags);
            m_treeItem.m_targetFpgaItem[i].m_hFlags = m_treeCtl.InsertItem(str, m_treeItem.m_targetFpgaItem[i].m_hFpga);
            if (fpgaInfo.Flags & ADMXRC3_FPGA_SPEEDVALID) {
              str.Format(_T("SPEEDVALID (0x%lX)"), (unsigned long)ADMXRC3_FPGA_SPEEDVALID);
              m_treeItem.m_targetFpgaItem[i].m_flagsItem.m_speed = m_treeCtl.InsertItem(str, m_treeItem.m_targetFpgaItem[i].m_hFlags);
              str.Format(_T("Speed: %s"), (const TCHAR*)fpgaInfo.SpeedGrade);
              m_treeItem.m_targetFpgaItem[i].m_hSpeedGrade = m_treeCtl.InsertItem(str, m_treeItem.m_targetFpgaItem[i].m_hFpga);
            }
            if (fpgaInfo.Flags & ADMXRC3_FPGA_STEPPINGVALID) {
              str.Format(_T("STEPPINGVALID (0x%lX)"), (unsigned long)ADMXRC3_FPGA_STEPPINGVALID);
              m_treeItem.m_targetFpgaItem[i].m_flagsItem.m_speed = m_treeCtl.InsertItem(str, m_treeItem.m_targetFpgaItem[i].m_hFlags);
              str.Format(_T("Stepping: %s"), (const TCHAR*)fpgaInfo.Stepping);
              m_treeItem.m_targetFpgaItem[i].m_hStepping = m_treeCtl.InsertItem(str, m_treeItem.m_targetFpgaItem[i].m_hFpga);
            }
          } else {
            str.Format(_T("%u  -  (Not present)"), i);
            m_treeItem.m_targetFpgaItem[i].m_hFpga = m_treeCtl.InsertItem(str, m_treeItem.m_hTargetFpga);
          }
        } else {
          str.Format(_T("%u  -  (Unable to determine: %s)"), i, ADMXRC3_GetStatusString(status, TRUE));
          m_treeItem.m_targetFpgaItem[i].m_hFpga = m_treeCtl.InsertItem(str, m_treeItem.m_hTargetFpga);
        }
      }

      // Window subtree
      for (unsigned int i = 0; i < cardInfoEx.NumWindow && i < m_maxNumWindow; i++) {
        const TCHAR* pTmp;

        status = ADMXRC3_GetWindowInfo(devInfo.m_hDevice, i, &windowInfo);
        if (status == ADMXRC3_SUCCESS) {
          pTmp = CAdmxrc3Help::getWindowDescription(i, cardInfoEx.Model);
          str.Format(_T("%u  -  %s"), i,
            (NULL != pTmp) ? pTmp : _T("Unknown"));
          m_treeItem.m_windowItem[i].m_hWindow = m_treeCtl.InsertItem(str, m_treeItem.m_hWindow);
          str.Format(_T("Bus base address: 0x%llX"), (unsigned long long)windowInfo.BusBase);
          m_treeItem.m_windowItem[i].m_hBusBase = m_treeCtl.InsertItem(str, m_treeItem.m_windowItem[i].m_hWindow);
          str.Format(_T("Bus limit address: 0x%llX"), (unsigned long long)(windowInfo.BusBase + windowInfo.BusSize - 1));
          m_treeItem.m_windowItem[i].m_hBusLimit = m_treeCtl.InsertItem(str, m_treeItem.m_windowItem[i].m_hWindow);
          str.Format(_T("Bus size: 0x%llX"), (unsigned long long)windowInfo.BusSize);
          m_treeItem.m_windowItem[i].m_hBusSize = m_treeCtl.InsertItem(str, m_treeItem.m_windowItem[i].m_hWindow);
          if (windowInfo.LocalSize) {
            str.Format(_T("Local base address: 0x%llX"), (unsigned long long)windowInfo.LocalBase);
            m_treeItem.m_windowItem[i].m_hLocalBase = m_treeCtl.InsertItem(str, m_treeItem.m_windowItem[i].m_hWindow);
            str.Format(_T("Local limit address: 0x%llX"), (unsigned long long)(windowInfo.LocalBase + windowInfo.LocalSize - 1));
            m_treeItem.m_windowItem[i].m_hLocalLimit = m_treeCtl.InsertItem(str, m_treeItem.m_windowItem[i].m_hWindow);
            str.Format(_T("Local size: 0x%llX"), (unsigned long long)windowInfo.LocalSize);
            m_treeItem.m_windowItem[i].m_hLocalSize = m_treeCtl.InsertItem(str, m_treeItem.m_windowItem[i].m_hWindow);
          }
          str.Format(_T("Virtual size: 0x%llX"), (unsigned long long)windowInfo.VirtualSize);
          m_treeItem.m_windowItem[i].m_hVirtualSize = m_treeCtl.InsertItem(str, m_treeItem.m_windowItem[i].m_hWindow);
        } else {
          str.Format(_T("%u  -  (Unable to determine: %s)"), i, ADMXRC3_GetStatusString(status, TRUE));
          m_treeItem.m_windowItem[i].m_hWindow = m_treeCtl.InsertItem(str, m_treeItem.m_hWindow);
        }
      }

      // Sensor subtree
      for (unsigned int i = 0; i < cardInfoEx.NumSensor && i < m_maxNumSensor; i++) {
        const TCHAR* pTmp;

        status = ADMXRC3_GetSensorInfo(devInfo.m_hDevice, i, &sensorInfo);
        if (status == ADMXRC3_SUCCESS) {
          str.Format(_T("%u  -  %s"), i, sensorInfo.Description);
          m_treeItem.m_sensorItem[i].m_hSensor = m_treeCtl.InsertItem(str, m_treeItem.m_hSensor);
          str.Format(_T("Description: %s"), sensorInfo.Description);
          m_treeItem.m_sensorItem[i].m_hDescription = m_treeCtl.InsertItem(str, m_treeItem.m_sensorItem[i].m_hSensor);
          pTmp = CAdmxrc3Help::getUnitString(sensorInfo.Unit);
          str.Format(_T("Unit: %lu (0x%lX) => %s"),
            (unsigned long)sensorInfo.Unit, (unsigned long)sensorInfo.Unit,
            (NULL != pTmp) ? pTmp : _T("unknown"));
          m_treeItem.m_sensorItem[i].m_hUnit = m_treeCtl.InsertItem(str, m_treeItem.m_sensorItem[i].m_hSensor);
          pTmp = CAdmxrc3Help::getDatatypeString(sensorInfo.DataType);
          str.Format(_T("Datatype: %lu (0x%lX) => %s"),
            (unsigned long)sensorInfo.DataType, (unsigned long)sensorInfo.DataType,
            (NULL != pTmp) ? pTmp : _T("unknown"));
          m_treeItem.m_sensorItem[i].m_hDatatype = m_treeCtl.InsertItem(str, m_treeItem.m_sensorItem[i].m_hSensor);
          str.Format(_T("Exponent: %d"), sensorInfo.Exponent);
          m_treeItem.m_sensorItem[i].m_hExponent = m_treeCtl.InsertItem(str, m_treeItem.m_sensorItem[i].m_hSensor);
          str.Format(_T("Error: %f"), sensorInfo.Error);
          m_treeItem.m_sensorItem[i].m_hError = m_treeCtl.InsertItem(str, m_treeItem.m_sensorItem[i].m_hSensor);
          str.Format(_T("Capabilities: 0x%lX"), (unsigned long)sensorInfo.Capabilities);
          m_treeItem.m_sensorItem[i].m_hCapabilities = m_treeCtl.InsertItem(str, m_treeItem.m_sensorItem[i].m_hSensor);
          if (sensorInfo.Capabilities & ADMXRC3_SENSOR_SWVALUE) {
            m_treeItem.m_sensorItem[i].m_capabilitiesItem.m_hSoftware = m_treeCtl.InsertItem(_T("Software sensor"), m_treeItem.m_sensorItem[i].m_hCapabilities);
          }
        } else {
          str.Format(_T("%u  -  (Unable to determine: %s)"), i, ADMXRC3_GetStatusString(status, TRUE));
          m_treeItem.m_sensorItem[i].m_hSensor = m_treeCtl.InsertItem(str, m_treeItem.m_hSensor);
        }
      }

#if !defined(HWSIM)
      // Flash subtree
      for (unsigned int i = 0; i < cardInfoEx.NumFlashBank && i < m_maxNumFlashBank; i++) {
        status = ADMXRC3_GetFlashInfo(devInfo.m_hDevice, i, &flashInfo);
        if (status == ADMXRC3_SUCCESS) {
          str.Format(_T("%u  -  %s"), i, flashInfo.Identifier);
          m_treeItem.m_flashItem[i].m_hFlash = m_treeCtl.InsertItem(str, m_treeItem.m_hFlash);
          str.Format(_T("Identifier: %s"), flashInfo.Identifier);
          m_treeItem.m_flashItem[i].m_hIdentifier = m_treeCtl.InsertItem(str, m_treeItem.m_flashItem[i].m_hFlash);
          str.Format(_T("Size: %llu (0x%llX) kiByte"), (unsigned long long)(flashInfo.Size >> 10), (unsigned long long)(flashInfo.Size >> 10));
          m_treeItem.m_flashItem[i].m_hSize = m_treeCtl.InsertItem(str, m_treeItem.m_flashItem[i].m_hFlash);
          str.Format(_T("Useable start address: 0x%llX"), (unsigned long long)flashInfo.UseableStart);
          m_treeItem.m_flashItem[i].m_hUseableStart = m_treeCtl.InsertItem(str, m_treeItem.m_flashItem[i].m_hFlash);
          str.Format(_T("Useable limit address: 0x%llX"), (unsigned long long)(flashInfo.UseableStart + flashInfo.UseableLength - 1ULL));
          m_treeItem.m_flashItem[i].m_hUseableLimit = m_treeCtl.InsertItem(str, m_treeItem.m_flashItem[i].m_hFlash);
          str.Format(_T("Useable length: 0x%llX"), (unsigned long long)flashInfo.UseableLength);
          m_treeItem.m_flashItem[i].m_hUseableLength = m_treeCtl.InsertItem(str, m_treeItem.m_flashItem[i].m_hFlash);
        } else {
          str.Format(_T("%u  -  (Unable to determine: %s)"), i, ADMXRC3_GetStatusString(status, TRUE));
          m_treeItem.m_flashItem[i].m_hFlash = m_treeCtl.InsertItem(str, m_treeItem.m_hFlash);
        }
      }

      // Memory bank subtree
      for (unsigned int i = 0; i < cardInfoEx.NumMemoryBank && i < m_maxNumMemoryBank; i++) {
        status = ADMXRC3_GetBankInfo(devInfo.m_hDevice, i, &bankInfo);
        if (status == ADMXRC3_SUCCESS) {
          if (bankInfo.Present) {
            UINT64 sizeByte = bankInfo.PhysicalSize * (bankInfo.PhysicalDataWidth >> 3);

            str.Format(_T("%u  -  %llu (0x%llX) kByte"), i, (unsigned long long)(sizeByte >> 10), (unsigned long long)(sizeByte >> 10));
            m_treeItem.m_memoryItem[i].m_hMemoryBank = m_treeCtl.InsertItem(str, m_treeItem.m_hMemory);
            str.Format(_T("Maximum frequency: %.1f MHz"), (double)bankInfo.MaximumFrequency / 1.0e6f);
            m_treeItem.m_memoryItem[i].m_hMaximumFrequency = m_treeCtl.InsertItem(str, m_treeItem.m_memoryItem[i].m_hMemoryBank);
            str.Format(_T("Minimum frequency: %.1f MHz"), (double)bankInfo.MinimumFrequency / 1.0e6f);
            m_treeItem.m_memoryItem[i].m_hMinimumFrequency = m_treeCtl.InsertItem(str, m_treeItem.m_memoryItem[i].m_hMemoryBank);
            str.Format(_T("Physical size: %llu (0x%llX) kWord"), (unsigned long long)(bankInfo.PhysicalSize >> 10), (unsigned long long)(bankInfo.PhysicalSize >> 10));
            m_treeItem.m_memoryItem[i].m_hPhysicalSize = m_treeCtl.InsertItem(str, m_treeItem.m_memoryItem[i].m_hMemoryBank);
            str.Format(_T("Physical non-ECC data width: %u bit(s)"), (unsigned int)bankInfo.PhysicalDataWidth);
            m_treeItem.m_memoryItem[i].m_hPhysicalDataWidth = m_treeCtl.InsertItem(str, m_treeItem.m_memoryItem[i].m_hMemoryBank);
            str.Format(_T("Physical ECC data width: %u bit(s)"), (unsigned int)bankInfo.PhysicalECCWidth);
            m_treeItem.m_memoryItem[i].m_hPhysicalEccWidth = m_treeCtl.InsertItem(str, m_treeItem.m_memoryItem[i].m_hMemoryBank);
            str.Format(_T("Physical data width: %u bit(s)"), (unsigned int)bankInfo.PhysicalWidth);
            m_treeItem.m_memoryItem[i].m_hPhysicalWidth = m_treeCtl.InsertItem(str, m_treeItem.m_memoryItem[i].m_hMemoryBank);
            str.Format(_T("Type mask: 0x%lX"), (unsigned int)bankInfo.TypeMask);
            m_treeItem.m_memoryItem[i].m_hTypeMask = m_treeCtl.InsertItem(str, m_treeItem.m_memoryItem[i].m_hMemoryBank);
            if (bankInfo.TypeMask & ADMXRC3_BANK_ZBT_FT) {
              m_treeItem.m_memoryItem[i].m_typeItem.m_hZbtFt = m_treeCtl.InsertItem(_T("Flowthrough ZBT"), m_treeItem.m_memoryItem[i].m_hTypeMask);
            }
            if (bankInfo.TypeMask & ADMXRC3_BANK_ZBT_P) {
              m_treeItem.m_memoryItem[i].m_typeItem.m_hZbtP = m_treeCtl.InsertItem(_T("Pipelined ZBT"), m_treeItem.m_memoryItem[i].m_hTypeMask);
            }
            if (bankInfo.TypeMask & ADMXRC3_BANK_SDRAM_DDR) {
              m_treeItem.m_memoryItem[i].m_typeItem.m_hSdramDdr = m_treeCtl.InsertItem(_T("DDR SDRAM"), m_treeItem.m_memoryItem[i].m_hTypeMask);
            }
            if (bankInfo.TypeMask & ADMXRC3_BANK_SSRAM_DDR2) {
              m_treeItem.m_memoryItem[i].m_typeItem.m_hSdramDdr = m_treeCtl.InsertItem(_T("DDR-II SSRAM"), m_treeItem.m_memoryItem[i].m_hTypeMask);
            }
            if (bankInfo.TypeMask & ADMXRC3_BANK_SDRAM_SDR) {
              m_treeItem.m_memoryItem[i].m_typeItem.m_hSdramDdr = m_treeCtl.InsertItem(_T("SDR SDRAM"), m_treeItem.m_memoryItem[i].m_hTypeMask);
            }
            if (bankInfo.TypeMask & ADMXRC3_BANK_SDRAM_DDR2) {
              m_treeItem.m_memoryItem[i].m_typeItem.m_hSdramDdr2 = m_treeCtl.InsertItem(_T("DDR-II SDRAM"), m_treeItem.m_memoryItem[i].m_hTypeMask);
            }
            if (bankInfo.TypeMask & ADMXRC3_BANK_SSRAM_QDR) {
              m_treeItem.m_memoryItem[i].m_typeItem.m_hSsramQdr = m_treeCtl.InsertItem(_T("QDR SSRAM"), m_treeItem.m_memoryItem[i].m_hTypeMask);
            }
            if (bankInfo.TypeMask & ADMXRC3_BANK_SDRAM_DDR3) {
              m_treeItem.m_memoryItem[i].m_typeItem.m_hSdramDdr3 = m_treeCtl.InsertItem(_T("DDR-3 SDRAM"), m_treeItem.m_memoryItem[i].m_hTypeMask);
            }
            str.Format(_T("Connectivity mask: 0x%lX"), (unsigned int)bankInfo.ConnectivityMask);
            m_treeItem.m_memoryItem[i].m_hConnectivityMask = m_treeCtl.InsertItem(str, m_treeItem.m_memoryItem[i].m_hMemoryBank);
            for (unsigned int j = 0; j < 32; j++) {
              if (bankInfo.ConnectivityMask & (0x1U << j)) {
                str.Format(_T("Target FPGA %u"), j);
                m_treeCtl.InsertItem(str, m_treeItem.m_memoryItem[i].m_hConnectivityMask);
              }
            }
          } else {
            str.Format(_T("%u  -  Not present"), i);
            m_treeItem.m_memoryItem[i].m_hMemoryBank = m_treeCtl.InsertItem(str, m_treeItem.m_memoryItem[i].m_hMemoryBank);
          }
          str.Format(_T("Present: %s"), bankInfo.Present ? _T("TRUE") : _T("FALSE"));
          m_treeItem.m_memoryItem[i].m_hPresent = m_treeCtl.InsertItem(str, m_treeItem.m_memoryItem[i].m_hMemoryBank);
        } else {
          str.Format(_T("%u  -  (Unable to determine: %s)"), i, ADMXRC3_GetStatusString(status, TRUE));
          m_treeItem.m_memoryItem[i].m_hMemoryBank = m_treeCtl.InsertItem(str, m_treeItem.m_memoryItem[i].m_hMemoryBank);
        }
      }
      // I/O module subtree
      for (unsigned int i = 0; i < cardInfoEx.NumModuleSite; i++) {
        status = ADMXRC3_GetModuleInfo(devInfo.m_hDevice, i, &moduleInfo);
        if (status == ADMXRC3_SUCCESS) {
          if (moduleInfo.Present) {
            if (!(moduleInfo.Flags & ADMXRC3_MODULE_LEGACYXRM)) {
              str.Format(_T("%u  -  %s #%s"), i, moduleInfo.Product, moduleInfo.SerialNumber);
              m_treeItem.m_moduleItem[i].m_hModule = m_treeCtl.InsertItem(str, m_treeItem.m_hModule);
              str.Format(_T("Product: %s"), moduleInfo.Product);
              m_treeItem.m_moduleItem[i].m_hProduct = m_treeCtl.InsertItem(str, m_treeItem.m_moduleItem[i].m_hModule);
              str.Format(_T("Part number: %s"), moduleInfo.PartNumber);
              m_treeItem.m_moduleItem[i].m_hPartNumber = m_treeCtl.InsertItem(str, m_treeItem.m_moduleItem[i].m_hModule);
              str.Format(_T("Manufacturer: %s"), moduleInfo.Manufacturer);
              m_treeItem.m_moduleItem[i].m_hManufacturer = m_treeCtl.InsertItem(str, m_treeItem.m_moduleItem[i].m_hModule);
              str.Format(_T("SerialNumber: %s"), moduleInfo.SerialNumber);
              m_treeItem.m_moduleItem[i].m_hSerialNumber = m_treeCtl.InsertItem(str, m_treeItem.m_moduleItem[i].m_hModule);
              str.Format(_T("I/O Voltage: %.2f"), moduleInfo.IoVoltage);
              m_treeItem.m_moduleItem[i].m_hIoVoltage = m_treeCtl.InsertItem(str, m_treeItem.m_moduleItem[i].m_hModule);
              str.Format(_T("Manufacture date: %lu minutes since 00:00 1/1/1996"), moduleInfo.ManufactureDate);
              m_treeItem.m_moduleItem[i].m_hManufactureDate = m_treeCtl.InsertItem(str, m_treeItem.m_moduleItem[i].m_hModule);
            } else {
              str.Format(_T("%u  -  Legacy XRM"), i);
              m_treeItem.m_moduleItem[i].m_hModule = m_treeCtl.InsertItem(str, m_treeItem.m_hModule);
            }
            str.Format(_T("Flags: 0x%lx"), moduleInfo.Flags);
            m_treeItem.m_moduleItem[i].m_hFlags = m_treeCtl.InsertItem(str, m_treeItem.m_moduleItem[i].m_hModule);
            if (moduleInfo.Flags & ADMXRC3_MODULE_LEGACYXRM) {
              str.Format(_T("LEGACYXRM (0x%x)"), ADMXRC3_MODULE_LEGACYXRM);
              m_treeItem.m_moduleItem[i].m_flagsItem.m_hLegacyXrm = m_treeCtl.InsertItem(str, m_treeItem.m_moduleItem[i].m_hFlags);
            }
            if (moduleInfo.Flags & ADMXRC3_MODULE_FORCE2V5) {
              str.Format(_T("FORCE2V5 (0x%x)"), ADMXRC3_MODULE_FORCE2V5);
              m_treeItem.m_moduleItem[i].m_flagsItem.m_hForce2V5 = m_treeCtl.InsertItem(str, m_treeItem.m_moduleItem[i].m_hFlags);
            }
            if (moduleInfo.Flags & ADMXRC3_MODULE_ROMERROR) {
              str.Format(_T("ROMERROR (0x%x)"), ADMXRC3_MODULE_ROMERROR);
              m_treeItem.m_moduleItem[i].m_flagsItem.m_hRomError = m_treeCtl.InsertItem(str, m_treeItem.m_moduleItem[i].m_hFlags);
            }
            if (moduleInfo.Flags & ADMXRC3_MODULE_CSUMERROR) {
              str.Format(_T("CSUMERROR (0x%x)"), ADMXRC3_MODULE_CSUMERROR);
              m_treeItem.m_moduleItem[i].m_flagsItem.m_hCSumError = m_treeCtl.InsertItem(str, m_treeItem.m_moduleItem[i].m_hFlags);
            }
          }
          str.Format(_T("Present: %s"), moduleInfo.Present ? _T("TRUE") : _T("FALSE"));
          m_treeItem.m_moduleItem[i].m_hPresent = m_treeCtl.InsertItem(str, m_treeItem.m_moduleItem[i].m_hModule);
        } else {
          str.Format(_T("%u  -  (Unable to determine: %s)"), i, ADMXRC3_GetStatusString(status, TRUE));
          m_treeItem.m_moduleItem[i].m_hModule = m_treeCtl.InsertItem(str, m_treeItem.m_moduleItem[i].m_hModule);
        }
      }
#endif
    }
  }

  m_treeCtl.SetRedraw(TRUE);
}

void
CDeviceInfoWnd::numDeviceChanged()
{
  unsigned int numDevice = m_pParent->getDeviceInfo().m_numDevice;

  if ((m_numDevice == 0) == (numDevice == 0)) {
    return;
  }

  m_treeCtl.SetRedraw(FALSE);
  m_treeCtl.DeleteAllItems();
  if (m_pParent->getDeviceInfo().m_numDevice) {
    m_treeItem.m_hApi = m_treeCtl.InsertItem(_T("API information"));
    for (unsigned int i = 0; i < ARRAY_LENGTH(m_treeItem.m_hApiItem); i++) {
      m_treeItem.m_hApiItem[i] = m_treeCtl.InsertItem(_T(""), m_treeItem.m_hApi);
    }
    m_treeCtl.Expand(m_treeItem.m_hApi, TVE_EXPAND);
    m_treeItem.m_hSummary = m_treeCtl.InsertItem(_T("Summary information"));
    for (unsigned int i = 0; i < ARRAY_LENGTH(m_treeItem.m_hSummaryItem); i++) {
      m_treeItem.m_hSummaryItem[i] = m_treeCtl.InsertItem(_T(""), m_treeItem.m_hSummary);
    }
    m_treeCtl.Expand(m_treeItem.m_hSummary, TVE_EXPAND);
    m_treeItem.m_hTargetFpga = m_treeCtl.InsertItem(_T("Target FPGA"));
    m_treeItem.m_hWindow = m_treeCtl.InsertItem(_T("Window"));
    m_treeItem.m_hFlash = m_treeCtl.InsertItem(_T("Flash memory bank"));
    m_treeItem.m_hMemory = m_treeCtl.InsertItem(_T("Memory bank"));
    m_treeItem.m_hSensor = m_treeCtl.InsertItem(_T("Sensor"));
    m_treeItem.m_hModule = m_treeCtl.InsertItem(_T("Module site"));
    deviceChanged();
  }
  m_treeCtl.SetRedraw(TRUE);

  m_numDevice = numDevice;
}

// -------------------------------------------------------------------------
// End of IDeviceInfoWnd interface
// -------------------------------------------------------------------------

CDeviceInfoWnd::CDeviceInfoWnd(
  ISysMonWnd* pParent,
  bool_t bVisible)
{
  const TCHAR* pWndClass = NULL;
  bool_t bSuccess;

  m_pParent = pParent;
  m_numDevice = 0;

  m_bgBrush.CreateSysColorBrush(COLOR_BTNFACE);
  m_font.CreateStockObject(DEFAULT_GUI_FONT);

  pWndClass = AfxRegisterWndClass(0, ::LoadCursor(NULL, IDC_ARROW), (HBRUSH)m_bgBrush, NULL);
  bSuccess = CreateEx(WS_EX_CONTROLPARENT, pWndClass, _T(""), WS_CHILD | (bVisible ? WS_VISIBLE : 0), 0, 0, 1, 1, *dynamic_cast<CWnd*>(m_pParent), NULL, NULL);
  ASSERT(bSuccess);
}

CDeviceInfoWnd::~CDeviceInfoWnd()
{
}

void
CDeviceInfoWnd::setTree(
  HTREEITEM ti,
  const TCHAR* pString)
{
  m_treeCtl.SetItemText(ti, pString);
}

int
CDeviceInfoWnd::OnCreate(
  LPCREATESTRUCT lpCreateStruct)
{
  RECT r = { 0, 0, 0, 0 };

  if (CWnd::OnCreate(lpCreateStruct) == -1) {
    return -1;
  }

  m_treeCtl.CreateEx(WS_EX_STATICEDGE, WS_CHILD | WS_VISIBLE | WS_TABSTOP | TVS_HASLINES | TVS_LINESATROOT | TVS_HASBUTTONS, r, this, IDCTL_DEVINFO_TREE);
  m_treeCtl.SetFont(&m_font);

  return 0;
}

void
CDeviceInfoWnd::OnSize(
  UINT nType,
  int cx,
  int cy)
{
  CWnd::OnSize(nType, cx, cy);

  m_treeCtl.MoveWindow(0, 0, cx, cy);
}
