#ifndef _ADATA_PLATFORM_TIME_H
#define _ADATA_PLATFORM_TIME_H

#include <vxWorks.h>
#include <sysLib.h>
#include <taskLib.h>
#include <tickLib.h>

typedef UINT64 admxrc3_time_t;

#if (_WRS_VXWORKS_MAJOR > 6)
# define timeGet() tick64Get()
#else
# define timeGet() tickGet()
#endif

static const admxrc3_time_t timeZero = 0U;

#if defined(_WRS_INLINE)
_WRS_INLINE admxrc3_time_t
#else
static __inline__ admxrc3_time_t
#endif
timeMake(
  long sec,
  long usec,
  bool_t bRoundUp)
{
  const unsigned int M = 1000000U;
  unsigned int rate = sysClkRateGet();
  admxrc3_time_t u;

  u = ((admxrc3_time_t)sec * M + usec) * rate;
  if (bRoundUp) {
    return (u + M - 1) / M;
  } else {
    return u / M;
  }
}

#define timeAfter(t1, t2) ((t1) > (t2))

#define timeBefore(t1, t2) ((t1) < (t2))

#define timeEqual(t1, t2) ((t1) == (t2))

#define timeAdd(t1, t2) ((t1) + (t2))

#define timeSubtract(t1, t2) ((t1) - (t2))

#if defined(_WRS_INLINE)
_WRS_INLINE double
#else
static __inline__ double
#endif
timeToDouble(
  admxrc3_time_t t)
{
  return (double)t / (double)sysClkRateGet();
}

#if defined(_WRS_INLINE)
_WRS_INLINE float
#else
static __inline__ float
#endif
timeToFloat(
  admxrc3_time_t t)
{
  return (float)t / (float)sysClkRateGet();
}

#if defined(_WRS_INLINE)
_WRS_INLINE admxrc3_time_t
#else
static __inline__ admxrc3_time_t
#endif
doubleToTime(
  double d)
{
  return (admxrc3_time_t)(d * (double)sysClkRateGet());
}

#if defined(_WRS_INLINE)
_WRS_INLINE admxrc3_time_t
#else
static __inline__ admxrc3_time_t
#endif
floatToTime(
  float f)
{
  return (admxrc3_time_t)(f * (float)sysClkRateGet());
}

#if defined(_WRS_INLINE)
_WRS_INLINE void
#else
static __inline__ void
#endif
sleepMs(
  unsigned int ms)
{
  const int k = 1000;
  INT64 ticks;

  ticks = ((INT64)ms * sysClkRateGet() + k - 1) / k;
  taskDelay((int)ticks); 
}

#endif
