--==============================================================================
--
--     File Name: adb3_target_types_pkg.vhd
--       Version: 0.1
--          Date: 22 Feb 2009
--         Model: Synthesisable
--
--  Dependencies: Set simulator resolution to ps for accurate simulation
--
--       Company: Alpha Data
--        Design: Package adb3_target_types_pkg
--
--   Description: Package of components used in target FPGAs using ADB3.
--
--   Limitations:
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2009 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 22 Feb 09: Initial design
--
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;

library work;
use work.adb3_ocp.all;

package adb3_target_types_pkg is

  type board_type_t    is (ADM_XRC_6TL,ADM_XRC_6T1);
  type target_use_t    is (SIM_OCP,SIM_MPTL,SYN_NGC);

  -- Define maximum number of OCP Channels
  constant MAX_DS_CHANNELS  : natural := 1;
  constant MAX_DMA_CHANNELS : natural := 4;
  constant MAX_DM_CHANNELS  : natural := 3;

  -- Define maximum number of memory banks
  constant MAX_MEM_BANKS    : natural := 4;

  -- Define maximum width of MPTL serial data interface
  constant MAX_MPTL_SER_WIDTH : natural := 8;

  -- Define maximum width of PN4/PN5 connectors
  constant MAX_PN4_GPIO_WIDTH : natural := 32;
  constant MAX_PN6_GPIO_WIDTH : natural := 38;
  constant MAX_XRM_MGT_WIDTH  : natural := 8;
  constant MAX_PN6_MGT_WIDTH  : natural := 8;

  -- Record type for double ended signals
  type std_logic_dbl_t is record
    p : std_logic;
    n : std_logic;
  end record;

end;


package body adb3_target_types_pkg is

end;