--==============================================================================
--
--     File Name: mptl128_interface_target_6t1_top.vhd
--       Version: 0.1
--          Date: 11 May 2010
--         Model: sim (Not Synthesisable)
--
--  Dependencies: Set simulator resolution to ps for accurate simulation
--
--       Company: Alpha Data
--        Design: target mptl interface wrapper
--
--   Description: Selects between the synthesis target mptl interfaces
--
--   Limitations:
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2010 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 11 May 2010: Initial design
--
--==============================================================================


library ieee;
use ieee.std_logic_1164.all;

library work;
use work.adb3_ocp.all;
use work.adb3_target_types_pkg.all;
use work.adb3_target_inc_pkg.all;
use work.adb3_target_pkg.all;

entity mptl128_interface_target_6t1_top is
port(
  -- OCP Interface
  ocp_clk                  : in    std_logic;
  direct_slave_m2s         : out   adb3_ocp_m2sT;
  direct_slave_s2m         : in    adb3_ocp_s2mT;
  dma_channels_m2s         : out   adb3_ocp_m2sT_array(0 to MAX_DMA_CHANNELS-1);
  dma_channels_s2m         : in    adb3_ocp_s2mT_array(0 to MAX_DMA_CHANNELS-1);
  direct_masters_m2s       : in    adb3_ocp_m2sT_array(0 to MAX_DM_CHANNELS-1);
  direct_masters_s2m       : out   adb3_ocp_s2mT_array(0 to MAX_DM_CHANNELS-1);
  dma_abort                : in    std_logic_vector(MAX_DMA_CHANNELS-1 downto 0);
  -- MPTL Interface
  mptl_b2t                 : in    mptl_pins_t;
  mptl_t2b                 : out   mptl_pins_t;
  mptl_clk                 : in    mptl_clk_t;
  mptl_clk_out             : out   std_logic;
  -- MPTL Initialisation
  ocp_ready                : in    std_logic;
  mptl_sb_b2t              : in    mptl_sb_b2t_t;
  mptl_sb_t2b              : out   mptl_sb_t2b_t;
  -- MPTL Sideband
  gpio_t2b                 : in    std_logic_vector(15 downto 0);
  gpio_b2t                 : out   std_logic_vector(15 downto 0));
end;


architecture synth of mptl128_interface_target_6t1_top is

  signal ds_int_m2s : adb3_ocp_m2sT;
  signal ds_int_s2m : adb3_ocp_s2mT;

  signal mptl_target_config_n     : std_logic;
  signal mptl_target_gtp_online_n : std_logic;
  signal mptl_bridge_gtp_online_n : std_logic;

begin

  syn_ngc_g : if (TARGET_USE = SYN_NGC) generate
    -- Direct connecton during synthesis
    direct_slave_m2s <= ds_int_m2s;
    ds_int_s2m       <= direct_slave_s2m;
  end generate;

  sim_mptl_g : if (TARGET_USE = SIM_MPTL) generate
    -- Define data during MPTL simulation
    direct_slave_m2s <= ds_int_m2s;
    ds_int_s2m       <= make_defined_s2m(direct_slave_s2m);
  end generate;


  -- Hold off MPTL target configured until all OCP clocks are stable (resets inactive)
  mptl_sb_t2b.mptl_target_configured_l <= mptl_target_config_n or not(ocp_ready);

  -- Connect MPTL initialisation signals
  mptl_sb_t2b.mptl_target_gtp_online_l <= mptl_target_gtp_online_n;
  mptl_bridge_gtp_online_n <= mptl_sb_b2t.mptl_bridge_gtp_online_l;


  mptl128_interface_target_6t1_i : mptl128_interface_target_6t1
  port map(
    -- OCP Interface
    ocp_clk                  => ocp_clk,                  -- in    std_logic
    direct_slave_m2s         => ds_int_m2s,               -- out   adb3_ocp_m2sT
    direct_slave_s2m         => ds_int_s2m,               -- in    adb3_ocp_s2mT
    dma_channels_m2s         => dma_channels_m2s(0 to 3), -- out   adb3_ocp_m2sT_array(0 to 3)
    dma_channels_s2m         => dma_channels_s2m(0 to 3), -- in    adb3_ocp_s2mT_array(0 to 3)
    dma_abort                => dma_abort(3 downto 0),    -- in    std_logic_vector(3 downto 0)
    -- MPTL Interface
    mptl_tx                  => mptl_t2b,                 -- out   mptl_pins_type
    mptl_rx                  => mptl_b2t,                 -- in    mptl_pins_type
    mptl_clk                 => mptl_clk,                 -- in    mptl_clk_type
    mptl_clk_out             => mptl_clk_out,             -- out   std_logic;
    -- MPTL Sideband Signals
    mptl_target_configured_n => mptl_target_config_n,     -- out   std_logic
    mptl_target_gtp_online_n => mptl_target_gtp_online_n, -- out   std_logic
    mptl_bridge_gtp_online_n => mptl_bridge_gtp_online_n, -- in    std_logic
    -- MPTL Sideband Interface
    gpio_t2b                 => gpio_t2b,                 -- in    std_logic_vector(15 downto 0)
    gpio_b2t                 => gpio_b2t);                -- out   std_logic_vector(15 downto 0)

end;