--==============================================================================
--
--     File Name: mptl_if_bridge_sim.vhd
--       Version: 0.1
--          Date: 23 Mar 2010
--         Model: sim (Not Synthesisable)
--
--  Dependencies: Set simulator resolution to ps for accurate simulation
--
--       Company: Alpha Data
--        Design: mptl_if_bridge for simulation only
--
--   Description: Simulation version of OCP to MPTL interface
--
--   Limitations:
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2010 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 23 Mar 10: Initial design
--
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

library work;
use work.adb3_ocp.all;
use work.adb3_target_types_pkg.all;
use work.adb3_target_inc_pkg.all;
use work.adb3_target_pkg.all;

entity mptl_if_bridge_sim is
port(
  -- OCP Interface
  ocp_clk            : in    std_logic;
  direct_slave_m2s   : in    adb3_ocp_m2sT;
  direct_slave_s2m   : out   adb3_ocp_s2mT;
  dma_channels_m2s   : in    adb3_ocp_m2sT_array(0 to MAX_DMA_CHANNELS-1);
  dma_channels_s2m   : out   adb3_ocp_s2mT_array(0 to MAX_DMA_CHANNELS-1);
  direct_masters_m2s : out   adb3_ocp_m2sT_array(0 to MAX_DM_CHANNELS-1);
  direct_masters_s2m : in    adb3_ocp_s2mT_array(0 to MAX_DM_CHANNELS-1);
  dma_abort          : out   std_logic_vector(MAX_DMA_CHANNELS-1 downto 0);
  -- MPTL Interface
  mptl_t2b           : in    mptl_pins_t;
  mptl_b2t           : out   mptl_pins_t;
  mptl_clk           : in    mptl_clk_t;
  -- MPTL Initialisation
  mptl_sb_t2b        : in    mptl_sb_t2b_t;
  mptl_sb_b2t        : out   mptl_sb_b2t_t;
  -- MPTL Sideband
  gpio_b2t           : in    std_logic_vector(15 downto 0);
  gpio_t2b           : out   std_logic_vector(15 downto 0);
  mptl_status        : out   std_logic_vector(63 downto 0);
  mptl_config        : in    std_logic_vector(31 downto 0));
end;


architecture rtl of mptl_if_bridge_sim is
  signal online_count : std_logic_vector(5 downto 0) := "111111";
  signal online_cnt_0 : boolean := false;

  signal target_config     : std_logic;
  signal target_gtp_online : std_logic;
  signal bridge_gtp_online : std_logic;

begin

  -- Connect MPTL initialisation signals
  mptl_sb_b2t.mptl_bridge_gtp_online_l <= not(bridge_gtp_online);

  -- Simulation interface inputs
  direct_slave_s2m   <= mptl_t2b.direct_slave_s2m;
  dma_channels_s2m   <= mptl_t2b.dma_channels_s2m;
  direct_masters_m2s <= mptl_t2b.direct_masters_m2s;
  gpio_t2b           <= mptl_t2b.gpio_t2b;

  -- Simulation interface outputs
  mptl_b2t.direct_slave_m2s   <= direct_slave_m2s;
  mptl_b2t.dma_channels_m2s   <= dma_channels_m2s;
  mptl_b2t.direct_masters_s2m <= direct_masters_s2m;
  mptl_b2t.gpio_b2t           <= gpio_b2t;

  -- Online outputs
  online_i : process(ocp_clk)
  begin
    if ocp_clk'event and ocp_clk = '1' then
      if online_count = "000000" then
        online_cnt_0 <= true;
      else
        online_count <= online_count-1;
        online_cnt_0 <= false;
      end if;

      if (online_cnt_0) then
        bridge_gtp_online <= '1';
      else
        bridge_gtp_online <= '0';
      end if;
    end if;
  end process;

  -- Status outputs
  mptl_status <= (others => '0');


  -- Connect dma abort
  -- dma_abort output using MPTL on 6T1
  inst_6t1_g : if (BOARD_TYPE = ADM_XRC_6T1) generate
    dma_abort <= mptl_t2b.dma_abort;
  end generate;

  -- dma_abort output directly on pins on 6TL
  inst_6tl_g : if (BOARD_TYPE = ADM_XRC_6TL) generate
    dma_abort <= not(mptl_sb_t2b.mptl_dma_abort_l);
  end generate;

end;