--==============================================================================
--
--     File Name: mptl_if_target_wrap.vhd
--       Version: 0.1
--          Date: 11 May 2010
--         Model: sim (Not Synthesisable)
--
--  Dependencies: Set simulator resolution to ps for accurate simulation
--
--       Company: Alpha Data
--        Design: target mptl interface wrapper
--
--   Description: Selects between the simulation and synthesis target mptl if
--
--   Limitations:
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2010 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 11 May 2010: Initial design
--
--==============================================================================


library ieee;
use ieee.std_logic_1164.all;

library work;
use work.adb3_ocp.all;
use work.adb3_target_types_pkg.all;
use work.adb3_target_inc_pkg.all;
use work.adb3_target_pkg.all;

entity mptl_if_target_wrap is
port(
  -- OCP Interface
  ocp_clk                  : in    std_logic;
  direct_slave_m2s         : out   adb3_ocp_m2sT;
  direct_slave_s2m         : in    adb3_ocp_s2mT;
  dma_channels_m2s         : out   adb3_ocp_m2sT_array(0 to MAX_DMA_CHANNELS-1);
  dma_channels_s2m         : in    adb3_ocp_s2mT_array(0 to MAX_DMA_CHANNELS-1);
  -- MPTL Interface
  mptl_t2b                 : out   mptl_pins_t;
  mptl_b2t                 : in    mptl_pins_t;
  mptl_clk                 : in    mptl_clk_t;
  -- MPTL Initialisation
  ocp_ready                : in    std_logic;
  mptl_sb_b2t              : in    mptl_sb_b2t_t;
  mptl_sb_t2b              : out   mptl_sb_t2b_t;
  -- MPTL Sideband
  gpio_t2b                 : in    std_logic_vector(15 downto 0);
  gpio_b2t                 : out   std_logic_vector(15 downto 0));
end;


architecture synth of mptl_if_target_wrap is

  signal unused_m2s   : adb3_ocp_m2sT_array(0 to MAX_DM_CHANNELS-1) := (others => ADB3_OCP_DEFAULT_M2S);
  signal dma_abort    : std_logic_vector(MAX_DMA_CHANNELS-1 downto 0);
  signal mptl_clk_out : std_logic;

begin

  -- dma_abort tied inactive (no OCP DM channels)
  dma_abort(DMA_CHANNELS-1 downto 0) <= (others => '0');

  inst_sim_ocp_g : if (TARGET_USE = SIM_OCP) generate
    -- direct OCP-OCP simulation interface
    mptl_if_target_sim_i : mptl_if_target_sim
    port map(
      -- OCP Interface
      ocp_clk                  => ocp_clk,           -- in    std_logic
      direct_slave_m2s         => direct_slave_m2s,  -- out   adb3_ocp_m2sT
      direct_slave_s2m         => direct_slave_s2m,  -- in    adb3_ocp_s2mT
      dma_channels_m2s         => dma_channels_m2s,  -- out   adb3_ocp_m2sT_array(0 to MAX_DMA_CHANNELS-1);
      dma_channels_s2m         => dma_channels_s2m,  -- in    adb3_ocp_s2mT_array(0 to MAX_DMA_CHANNELS-1);
      direct_masters_m2s       => unused_m2s,        -- in    adb3_ocp_m2sT_array(0 to MAX_DM_CHANNELS-1);
      direct_masters_s2m       => open,              -- out   adb3_ocp_s2mT_array(0 to MAX_DM_CHANNELS-1);
      dma_abort                => dma_abort,         -- in    std_logic_vector(MAX_DMA_CHANNELS-1 downto 0);
      -- MPTL Interface
      mptl_b2t                 => mptl_b2t,          -- in    mptl_pins_t;
      mptl_t2b                 => mptl_t2b,          -- out   mptl_pins_t;
      mptl_clk                 => mptl_clk,          -- in    mptl_clk_t;
      mptl_clk_out             => mptl_clk_out,      -- out   std_logic;
      -- MPTL Initialisation
      ocp_ready                => ocp_ready,         -- in    std_logic;
      mptl_sb_b2t              => mptl_sb_b2t,       -- in    mptl_sb_b2t_t;
      mptl_sb_t2b              => mptl_sb_t2b,       -- out   mptl_sb_t2b_t;
      -- MPTL Sideband
      gpio_t2b                 => gpio_t2b,          -- in    std_logic_vector(15 downto 0)
      gpio_b2t                 => gpio_b2t);         -- out   std_logic_vector(15 downto 0)
  end generate;

  inst_syn_6t1_g : if (TARGET_USE /= SIM_OCP)and(BOARD_TYPE = ADM_XRC_6T1) generate
    mptl128_interface_target_6t1_top_i : mptl128_interface_target_6t1_top
    port map(
      -- OCP Interface
      ocp_clk                  => ocp_clk,           -- in    std_logic
      direct_slave_m2s         => direct_slave_m2s,  -- out   adb3_ocp_m2sT
      direct_slave_s2m         => direct_slave_s2m,  -- in    adb3_ocp_s2mT
      dma_channels_m2s         => dma_channels_m2s,  -- out   adb3_ocp_m2sT_array(0 to MAX_DMA_CHANNELS-1)
      dma_channels_s2m         => dma_channels_s2m,  -- in    adb3_ocp_s2mT_array(0 to MAX_DMA_CHANNELS-1)
      direct_masters_m2s       => unused_m2s,        -- in    adb3_ocp_m2sT_array(0 to MAX_DM_CHANNELS-1);
      direct_masters_s2m       => open,              -- out   adb3_ocp_s2mT_array(0 to MAX_DM_CHANNELS-1);
      dma_abort                => dma_abort,         -- in    std_logic_vector(MAX_DMA_CHANNELS-1 downto 0)
      -- MPTL Interface
      mptl_b2t                 => mptl_b2t,          -- in    mptl_pins_type
      mptl_t2b                 => mptl_t2b,          -- out   mptl_pins_type
      mptl_clk                 => mptl_clk,          -- in    mptl_clk_type
      mptl_clk_out             => mptl_clk_out,      -- out   std_logic;
      -- MPTL Initialisation
      ocp_ready                => ocp_ready,         -- in    std_logic;
      mptl_sb_b2t              => mptl_sb_b2t,       -- in    mptl_sb_b2t_t;
      mptl_sb_t2b              => mptl_sb_t2b,       -- out   mptl_sb_t2b_t;
      -- MPTL Sideband
      gpio_t2b                 => gpio_t2b,          -- in    std_logic_vector(15 downto 0)
      gpio_b2t                 => gpio_b2t);         -- out   std_logic_vector(15 downto 0)
  end generate;

  inst_syn_6tl_g : if (TARGET_USE /= SIM_OCP)and(BOARD_TYPE = ADM_XRC_6TL) generate
    mptl64par_interface_target_6tl_top_i : mptl64par_interface_target_6tl_top
    port map(
      -- OCP Interface
      ocp_clk                  => ocp_clk,           -- in    std_logic
      direct_slave_m2s         => direct_slave_m2s,  -- out   adb3_ocp_m2sT
      direct_slave_s2m         => direct_slave_s2m,  -- in    adb3_ocp_s2mT
      dma_channels_m2s         => dma_channels_m2s,  -- out   adb3_ocp_m2sT_array(0 to MAX_DMA_CHANNELS-1)
      dma_channels_s2m         => dma_channels_s2m,  -- in    adb3_ocp_s2mT_array(0 to MAX_DMA_CHANNELS-1)
      direct_masters_m2s       => unused_m2s,        -- in    adb3_ocp_m2sT_array(0 to MAX_DM_CHANNELS-1);
      direct_masters_s2m       => open,              -- out   adb3_ocp_s2mT_array(0 to MAX_DM_CHANNELS-1);
      dma_abort                => dma_abort,         -- in    std_logic_vector(MAX_DMA_CHANNELS-1 downto 0)
      -- MPTL Interface
      mptl_b2t                 => mptl_b2t,          -- in    mptl_pins_type
      mptl_t2b                 => mptl_t2b,          -- out   mptl_pins_type
      mptl_clk                 => mptl_clk,          -- in    mptl_clk_type
      mptl_clk_out             => mptl_clk_out,      -- out   std_logic
      -- MPTL Initialisation
      ocp_ready                => ocp_ready,         -- in    std_logic;
      mptl_sb_b2t              => mptl_sb_b2t,       -- in    mptl_sb_b2t_t;
      mptl_sb_t2b              => mptl_sb_t2b,       -- out   mptl_sb_t2b_t;
      -- MPTL Sideband
      gpio_t2b                 => gpio_t2b,          -- in    std_logic_vector(15 downto 0)
      gpio_b2t                 => gpio_b2t);         -- out   std_logic_vector(15 downto 0)
  end generate;

end;