--==============================================================================
--
--     File Name: test_uber_dm.vhd
--       Version: 0.1
--          Date: 17 Mar 2010
--         Model: Behavioral (NOT SYNTHESISABLE)
--
--  Dependencies: Set simulator resolution to ps for accurate simulation
--
--       Company: Alpha Data
--        Design: ADB3 Target Design Test: Direct Master
--
--   Description: Provides bridge Direct Master interface
--
--   Limitations:
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2010 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 17 Mar 09: Initial design
--
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;

library work;
use work.adb3_ocp.all;
use work.adb3_target_types_pkg.all;
use work.adb3_target_pkg.all;
use work.uber_pkg.all;
use work.uber_tb_pkg.all;

entity test_uber_dm is
port(
  -- OCP Interface
  ocp_clk            : in    std_logic;
  direct_masters_m2s : in    adb3_ocp_m2sT_array(0 to MAX_DM_CHANNELS-1);
  direct_masters_s2m : out   adb3_ocp_s2mT_array(0 to MAX_DM_CHANNELS-1));
end test_uber_dm;

architecture test of test_uber_dm is

  constant BE_WIDTH  : natural := DM_DATA_WIDTH/8;

  -- Register interface signals
  signal ds_d  : std_logic_vector(DM_DATA_WIDTH-1 downto 0);
  signal ds_q  : std_logic_vector(DM_DATA_WIDTH-1 downto 0);
  signal ds_a  : std_logic_vector(DM_ADDR_WIDTH-1 downto 0);
  signal ds_w  : std_logic;
  signal ds_r  : std_logic;
  signal ds_be : std_logic_vector(BE_WIDTH-1 downto 0);

  signal master_reg : std_logic_vector(DM_DATA_WIDTH-1 downto 0);

begin

  adb3_ocp_simple_bus_if_i : adb3_ocp_simple_bus_if
  generic map(
    addr_width   => DM_ADDR_WIDTH,   -- natural range 1 to ADB3_OCP_ADDR_WIDTH
    data_width   => DM_DATA_WIDTH)   -- natural range 8 to ADB3_OCP_DATA_WIDTH
  port map(
    -- OCP Interface
    ocp_rst => '0',                             -- in    std_logic
    ocp_clk => ocp_clk,                         -- in    std_logic
    ocp_m2s => direct_masters_m2s(DM_CHANNEL),  -- in    adb3_ocp_m2sT
    ocp_s2m => direct_masters_s2m(DM_CHANNEL),  -- out   adb3_ocp_s2mT
    -- Simple bus interface
    d       => ds_d,        -- out   std_logic_vector(data_width-1 downto 0)
    q       => ds_q,        -- in    std_logic_vector(data_width-1 downto 0)
    a       => ds_a,        -- out   std_logic_vector(addr_width-1 downto 0)
    w       => ds_w,        -- out   std_logic
    r       => ds_r,        -- out   std_logic
    we      => ds_be);      -- out   std_logic_vector((data_width/8)-1 downto 0))

  -- Update write registers
  wr_regs : process(ocp_clk)
    variable u,l : natural;
  begin
    if rising_edge(ocp_clk) then
      for i in 0 to BE_WIDTH-1 loop
        u := 8*(i+1)-1;
        l := 8*i;
        if (ds_w = '1')and(ds_be(i)='1') then
          if (ds_a = HOST_MASTER_REG_ADDR(DM_ADDR_WIDTH-1 downto 0)) then
            master_reg(u downto l) <= ds_d(u downto l);
          end if;
        end if;
      end loop;
    end if;
  end process;

  -- Multiplex read registers
  rd_regs : process(ocp_clk)
  begin
    if rising_edge(ocp_clk) then
      if (ds_a = HOST_MASTER_REG_ADDR(DM_ADDR_WIDTH-1 downto 0)) then
        ds_q <= master_reg;
      else
        ds_q <= (others => '-');
      end if;
    end if;
  end process;

end test;