--==============================================================================
--
--     File Name: test_uber_ds.vhd
--       Version: 0.1
--          Date: 17 Mar 2010
--         Model: Behavioral (NOT SYNTHESISABLE)
--
--  Dependencies: Set simulator resolution to ps for accurate simulation
--
--       Company: Alpha Data
--        Design: ADB3 Target Design Test: Direct Slave
--
--   Description: Provides bridge Direct Slave interface
--
--   Limitations:
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2010 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 17 Mar 09: Initial design
--
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;
use ieee.std_logic_misc.all;

library work;
use work.adb3_ocp.all;
use work.adb3_target_types_pkg.all;
use work.adb3_target_inc_pkg.all;
use work.adb3_target_pkg.all;
use work.adb3_target_tb_pkg.all;
use work.uber_pkg.all;
use work.today_pkg.all;
use work.uber_tb_pkg.all;

entity test_uber_ds is
port(
  -- OCP Interface
  ocp_clk          : in    std_logic;
  direct_slave_m2s : out   adb3_ocp_m2sT;
  direct_slave_s2m : in    adb3_ocp_s2mT;
  -- Test interface
  mptl_online_long : in    std_logic;
  ds_comp          : out   ds_comp_t;
  ds_pass          : out   ds_pass_t);
end test_uber_ds;

architecture test of test_uber_ds is

  constant PN4_DATA : std_logic_vector(31 downto 0) := X"AABBCCDD";
  constant PN6_DATA : std_logic_vector(63 downto 0) := X"AAAABBBBCCCCDDDD";

  signal ds_m2s : adb3_ocp_m2sT := ADB3_OCP_DEFAULT_M2S;
  signal ds_s2m : adb3_ocp_s2mT;

begin
  ---------------------------------
  -- MPTL Direct Slave interface --
  ---------------------------------

  -- Connect direct slave input
  ds_s2m <= direct_slave_s2m;

  direct_slave_p : process
    variable reg_data_wr     : byte_vector_t(0 to 127);
    variable reg_data_rd     : byte_vector_t(0 to 127);
    variable reg_be          : byte_enable_t(0 to 127);
    variable reg_addr        : ds_addr_s;
    variable reg_xfer        : natural;
    variable master_idle     : std_logic;
    variable master_timeout  : natural := 64;
    variable master_cnt      : natural := 0;
    variable master_passed   : boolean := true;
    variable clock_running   : std_logic;
    variable ck_passed       : boolean := true;
    variable io_data_ls      : std_logic_vector(31 downto 0);
    variable io_data_ms      : std_logic_vector(31 downto 0);
    variable io_data_x2      : std_logic_vector(63 downto 0);
    variable f_io_passed     : boolean := true;
    variable r_io_passed     : boolean := true;
    variable int_set         : std_logic_vector(31 downto 0);
    variable int_stat        : std_logic_vector(31 downto 0);
    variable int_passed      : boolean := true;
    variable info_passed     : boolean := true;
    variable bram_passed     : boolean := true;

  begin
    wait until (mptl_online_long = '1');  -- OCP commands ignored until MPTL online
    ds_comp.simple_complete  <= false;
    ds_comp.master_complete  <= false;
    ds_comp.clock_complete   <= false;
    ds_comp.frontio_complete <= false;
    ds_comp.reario_complete  <= false;
    ds_comp.int_complete     <= false;
    ds_comp.info_complete    <= false;
    ds_comp.bram_complete    <= false;

    ds_pass.simple_passed    <= true;
    ds_pass.master_passed    <= true;
    ds_pass.clock_passed     <= true;
    ds_pass.frontio_passed   <= true;
    ds_pass.reario_passed    <= true;
    ds_pass.int_passed       <= true;
    ds_pass.info_passed      <= true;
    ds_pass.bram_passed      <= true;


    ---------------------------------------------
    -- Simple Test (32-bit register interface) --
    ---------------------------------------------
    reg_be(0 to 3) := conv_byte_enable("1111");

    -- Write a pattern to register
    reg_addr := SIMPLE_ADDR_BASE + SIMPLE_WDATA_REG_ADDR;
    reg_data_wr(0 to 3) := conv_byte_vector(X"cafeface");
    adb3_target_sim_write_reg32(reg_addr,10,reg_be(0 to 3),reg_data_wr(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Wrote simple WDATA " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_wr(0 to 3)) &
           " with enable 0b" & conv_string(reg_be(0 to 3)) &
           " to byte address " & conv_string(reg_addr);

    -- Read back nibble reversed pattern from register
    reg_addr := SIMPLE_ADDR_BASE + SIMPLE_RDATA_REG_ADDR;
    adb3_target_sim_read_reg32(reg_addr,11,reg_data_rd(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Read simple RDATA " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_rd(0 to 3)) &
           " from byte address " & conv_string(reg_addr);

    ds_comp.simple_complete <= true;
    if (reg_data_rd(0 to 3) = conv_byte_vector(X"ecafefac")) then
      assert false report "Test Simple completed: PASSED." severity note;
    else
      assert false report "Test Simple completed: FAILED." severity error;
      ds_pass.simple_passed <= false;
    end if;


    if (MASTER_USE = MASTER_ON) then
      ---------------------------------------------
      -- Master Test (32-bit register interface) --
      ---------------------------------------------
      reg_be(0 to 3) := conv_byte_enable("1111");

      -- Read OOR
      reg_addr := BRAM_ADDR_BASE + BRAM_ADDR_MASK + 1;
      adb3_target_sim_read(reg_addr,12,reg_data_rd(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
      report "Read OOR Addr 0 " & integer'image(reg_xfer) &
             " bytes 0x" & conv_string_hex(reg_data_rd(0 to 3)) &
             " from byte address " & conv_string(reg_addr);

      if (reg_data_rd(0 to 3) /= conv_byte_vector(X"DEADC0DE")) then
        master_passed := false;
      end if;

      -- Write direct master write data
      reg_addr := MASTER_ADDR_BASE + MASTER_WDATA_REG_ADDR;
      reg_data_wr(0 to 3) := conv_byte_vector(X"76543210");  -- 32-bit data
      adb3_target_sim_write_reg32(reg_addr,12,reg_be(0 to 3),reg_data_wr(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
      report "Wrote master WDATA " & integer'image(reg_xfer) &
             " bytes 0x" & conv_string_hex(reg_data_wr(0 to 3)) &
             " with enable 0b" & conv_string(reg_be(0 to 3)) &
             " to byte address " & conv_string(reg_addr);

      -- Write direct master configuration (1 = write, 0 = read)
      reg_addr := MASTER_ADDR_BASE + MASTER_SEL_REG_ADDR;
      reg_data_wr(0 to 3) := conv_byte_vector(X"0000000" & "0001");  -- bit 0 = write
      adb3_target_sim_write_reg32(reg_addr,13,reg_be(0 to 3),reg_data_wr(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
      report "Wrote master SEL " & integer'image(reg_xfer) &
             " bytes 0x" & conv_string_hex(reg_data_wr(0 to 3)) &
             " with enable 0b" & conv_string(reg_be(0 to 3)) &
             " to byte address " & conv_string(reg_addr);

      -- Write direct master address and trigger transfer
      reg_addr := MASTER_ADDR_BASE + MASTER_ADDR_REG_ADDR;
      reg_data_wr(0 to 3) := conv_byte_vector(HOST_MASTER_REG_ADDR(DM_DATA_WIDTH-1 downto 0));
      adb3_target_sim_write_reg32(reg_addr,14,reg_be(0 to 3),reg_data_wr(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
      report "Wrote master ADDR " & integer'image(reg_xfer) &
             " bytes 0x" & conv_string_hex(reg_data_wr(0 to 3)) &
             " with enable 0b" & conv_string(reg_be(0 to 3)) &
             " to byte address " & conv_string(reg_addr);

      -- Write direct master configuration (1 = write, 0 = read)
      reg_addr := MASTER_ADDR_BASE + MASTER_SEL_REG_ADDR;
      reg_data_wr(0 to 3) := conv_byte_vector(X"0000000" & "0000");  -- bit 0 = read
      adb3_target_sim_write_reg32(reg_addr,15,reg_be(0 to 3),reg_data_wr(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
      report "Wrote master SEL " & integer'image(reg_xfer) &
             " bytes 0x" & conv_string_hex(reg_data_wr(0 to 3)) &
             " with enable 0b" & conv_string(reg_be(0 to 3)) &
             " to byte address " & conv_string(reg_addr);

      -- Write direct master address and trigger transfer
      reg_addr := MASTER_ADDR_BASE + MASTER_ADDR_REG_ADDR;
      reg_data_wr(0 to 3) := conv_byte_vector(HOST_MASTER_REG_ADDR(DM_DATA_WIDTH-1 downto 0));
      adb3_target_sim_write_reg32(reg_addr,16,reg_be(0 to 3),reg_data_wr(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
      report "Wrote master ADDR " & integer'image(reg_xfer) &
             " bytes 0x" & conv_string_hex(reg_data_wr(0 to 3)) &
             " with enable 0b" & conv_string(reg_be(0 to 3)) &
             " to byte address " & conv_string(reg_addr);

      master_idle := '0';
      while ((master_idle = '0')and(master_cnt < master_timeout)) loop
        -- Read direct master status
        reg_addr := MASTER_ADDR_BASE + MASTER_STAT_REG_ADDR;
        adb3_target_sim_read_reg32(reg_addr,17,reg_data_rd(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
        master_idle         := reg_data_rd(0)(0);
        master_cnt          := master_cnt + 1;
      end loop;

      if (master_cnt < master_timeout) then
        report "Direct Master transfer complete (" & std_logic'image(master_idle) & ")";
      else
        report "Direct Master transfer timeout (" & std_logic'image(master_idle) & ")";
        master_passed := false;
      end if;

      -- Read direct master read data
      reg_addr := MASTER_ADDR_BASE + MASTER_RDATA_REG_ADDR;
      adb3_target_sim_read_reg32(reg_addr,18,reg_data_rd(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
      report "Read " & integer'image(reg_xfer) &
             " bytes 0x" & conv_string_hex(reg_data_rd(0 to 3)) &
             " from byte address " & conv_string(reg_addr);

      if (reg_data_rd(0 to 3) /= conv_byte_vector(X"76543210")) then
        master_passed := false;
      end if;

      ds_comp.master_complete <= true;
      if (master_passed) then
        assert false report "Test Master completed: PASSED." severity note;
      else
        assert false report "Test Master completed: FAILED." severity error;
        ds_pass.master_passed <= false;
      end if;
    end if;


    ------------------------------------------------------------------
    -- Clock Frequency Measurement Test (32-bit register interface) --
    ------------------------------------------------------------------
    reg_be(0 to 3) := conv_byte_enable("1111");

    -- Read OOR
    reg_addr := BRAM_ADDR_BASE + BRAM_ADDR_MASK + 1;
    adb3_target_sim_read(reg_addr,19,reg_data_rd(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Read OOR Addr 0 " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_rd(0 to 3)) &
           " from byte address " & conv_string(reg_addr);

    if (reg_data_rd(0 to 3) /= conv_byte_vector(X"DEADC0DE")) then
      ck_passed := false;
    end if;

    -- Clear all flags
    reg_addr := CLK_READ_ADDR_BASE + CLK_READ_CTRL_REG_ADDR;
    reg_data_wr(0 to 3) := conv_byte_vector(X"80000000");
    adb3_target_sim_write_reg32(reg_addr,19,reg_be(0 to 3),reg_data_wr(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Wrote Clear All CTRL " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_wr(0 to 3)) &
           " with enable 0b" & conv_string(reg_be(0 to 3)) &
           " to byte address " & conv_string(reg_addr);

    -- Select pll_reg_clk
    reg_addr := CLK_READ_ADDR_BASE + CLK_READ_SEL_REG_ADDR;
    reg_data_wr(0 to 3) := conv_byte_vector(X"000000" & "000" & PLL_REG_CLK_SEL);
    adb3_target_sim_write_reg32(reg_addr,20,reg_be(0 to 3),reg_data_wr(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Wrote PLL_REG_CLK_SEL SEL " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_wr(0 to 3)) &
           " with enable 0b" & conv_string(reg_be(0 to 3)) &
           " to byte address " & conv_string(reg_addr);

    -- Wait until clock register has been updated (msb = 1)
    -- Then read clock frequency
    clock_running := '0';
    while (clock_running = '0') loop
      reg_addr := CLK_READ_ADDR_BASE + CLK_READ_STAT_REG_ADDR;
      adb3_target_sim_read_reg32(reg_addr,21,reg_data_rd(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
      clock_running := reg_data_rd(3)(7);
    end loop;

    -- Read frequency data
    reg_addr := CLK_READ_ADDR_BASE + CLK_READ_FREQ_REG_ADDR;
    adb3_target_sim_read_reg32(reg_addr,22,reg_data_rd(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Read PLL_REG_CLK FREQ " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_rd(0 to 3)) &
           " from byte address " & conv_string(reg_addr);

    report "Expected freq = " & natural'image(CLK_80_FREQ) & " or " & natural'image(CLK_80_FREQ+1) & " MHz";
    report "Actual freq = " & integer'image(to_integer(unsigned(reg_data_rd(0)))) & " MHz";

    -- Check for truncated  and +1 frequency values
    if ((to_integer(unsigned(reg_data_rd(0))) /= CLK_80_FREQ)and
        (to_integer(unsigned(reg_data_rd(0))) /= (CLK_80_FREQ+1)))then
      ck_passed := false;
    end if;

    -- Select pll_ref_clk
    reg_addr := CLK_READ_ADDR_BASE + CLK_READ_SEL_REG_ADDR;
    reg_data_wr(0 to 3) := conv_byte_vector(X"000000" & "000" & PLL_REF_CLK_SEL);
    adb3_target_sim_write_reg32(reg_addr,23,reg_be(0 to 3),reg_data_wr(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Wrote PLL_REF_CLK_SEL SEL " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_wr(0 to 3)) &
           " with enable 0b" & conv_string(reg_be(0 to 3)) &
           " to byte address " & conv_string(reg_addr);

    -- Wait until clock register has been updated (msb = 1)
    -- Then read clock frequency
    clock_running := '0';
    while (clock_running = '0') loop
      reg_addr := CLK_READ_ADDR_BASE + CLK_READ_STAT_REG_ADDR;
      adb3_target_sim_read_reg32(reg_addr,24,reg_data_rd(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
      clock_running       := reg_data_rd(3)(7);
    end loop;

    -- Read frequency data
    reg_addr := CLK_READ_ADDR_BASE + CLK_READ_FREQ_REG_ADDR;
    adb3_target_sim_read_reg32(reg_addr,25,reg_data_rd(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Read PLL_REF_CLK FREQ " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_rd(0 to 3)) &
           " from byte address " & conv_string(reg_addr);

    report "Expected freq = " & natural'image(CLK_200_FREQ) & " MHz";
    report "Actual freq = " & integer'image(to_integer(unsigned(reg_data_rd(0)))) & " MHz";

    if (to_integer(unsigned(reg_data_rd(0))) /= CLK_200_FREQ) then
      ck_passed := false;
    end if;

    -- Select MGT 113 clk0
    reg_addr := CLK_READ_ADDR_BASE + CLK_READ_SEL_REG_ADDR;
    reg_data_wr(0 to 3) := conv_byte_vector(X"000000" & "000" & MGT113_CLK0_SEL);
    adb3_target_sim_write_reg32(reg_addr,26,reg_be(0 to 3),reg_data_wr(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Wrote MGT113_CLK0_SEL SEL " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_wr(0 to 3)) &
           " with enable 0b" & conv_string(reg_be(0 to 3)) &
           " to byte address " & conv_string(reg_addr);

    -- Wait until clock register has been updated (msb = 1)
    -- Then read clock frequency
    clock_running := '0';
    while (clock_running = '0') loop
      reg_addr := CLK_READ_ADDR_BASE + CLK_READ_STAT_REG_ADDR;
      adb3_target_sim_read_reg32(reg_addr,27,reg_data_rd(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
      clock_running       := reg_data_rd(3)(7);
    end loop;

    -- Read frequency data
    reg_addr := CLK_READ_ADDR_BASE + CLK_READ_FREQ_REG_ADDR;
    adb3_target_sim_read_reg32(reg_addr,28,reg_data_rd(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Read MGT113_CLK0 FREQ " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_rd(0 to 3)) &
           " from byte address " & conv_string(reg_addr);

    if (BOARD_TYPE = ADM_XRC_6TL) then
      report "Expected freq = " & natural'image(CLK_100_FREQ) & " MHz";
    end if;
    if (BOARD_TYPE = ADM_XRC_6T1) then
      report "Expected freq = " & natural'image(CLK_250_FREQ) & " MHz";
    end if;
    report "Actual freq = " & integer'image(to_integer(unsigned(reg_data_rd(0)))) & " MHz";

    if (BOARD_TYPE = ADM_XRC_6TL)and(to_integer(unsigned(reg_data_rd(0))) /= CLK_100_FREQ) then
      ck_passed := false;
    end if;
    if (BOARD_TYPE = ADM_XRC_6T1)and(to_integer(unsigned(reg_data_rd(0))) /= CLK_250_FREQ) then
      ck_passed := false;
    end if;

    ds_comp.clock_complete <= true;
    if (ck_passed) then
      assert false report "Test Clock Read completed: PASSED." severity note;
    else
      assert false report "Test Clock Read completed: FAILED." severity error;
      ds_pass.clock_passed <= false;
    end if;


    -----------------------------------------------
    -- XRM GPIO Test (32-bit register interface) --
    -----------------------------------------------
    reg_be(0 to 3) := conv_byte_enable("1111");

    -- Read OOR
    reg_addr := BRAM_ADDR_BASE + BRAM_ADDR_MASK + 1;
    adb3_target_sim_read(reg_addr,12,reg_data_rd(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Read OOR Addr 0 " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_rd(0 to 3)) &
           " from byte address " & conv_string(reg_addr);

    if (reg_data_rd(0 to 3) /= conv_byte_vector(X"DEADC0DE")) then
      f_io_passed := false;
    end if;

    -- Write io data
    io_data_ls := X"76543210";

    reg_addr := IO_ADDR_BASE + XRM_GPIO_DD_DATA_REG_ADDR;
    reg_data_wr(0 to 3) := conv_byte_vector(io_data_ls);
    adb3_target_sim_write_reg32(reg_addr,29,reg_be(0 to 3),reg_data_wr(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Wrote XRM_GPIO_DA DATA " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_wr(0 to 3)) &
           " with enable 0b" & conv_string(reg_be(0 to 3)) &
           " to byte address " & conv_string(reg_addr);

    -- Write io tri-state enable
    reg_addr := IO_ADDR_BASE + XRM_GPIO_DD_TRI_REG_ADDR;
    reg_data_wr(0 to 3) := conv_byte_vector(X"00000000");
    adb3_target_sim_write_reg32(reg_addr,30,reg_be(0 to 3),reg_data_wr(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Wrote XRM_GPIO_DA TRI " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_wr(0 to 3)) &
           " with enable 0b" & conv_string(reg_be(0 to 3)) &
           " to byte address " & conv_string(reg_addr);

    -- Wait for data to get to outputs
    adb3_wait_cycles(50,ocp_clk);

    -- Read io data
    reg_addr := IO_ADDR_BASE + XRM_GPIO_DD_DATA_REG_ADDR;
    adb3_target_sim_read_reg32(reg_addr,31,reg_data_rd(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Read XRM_GPIO_DA DATA " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_rd(0 to 3)) &
           " from byte address " & conv_string(reg_addr);

    if (conv_vector(reg_data_rd(0 to 3)) /= io_data_ls) then
      f_io_passed := false;
    end if;

    -- Write io tri-state disable
    reg_addr := IO_ADDR_BASE + XRM_GPIO_DD_TRI_REG_ADDR;
    reg_data_wr(0 to 3) := conv_byte_vector(X"FFFFFFFF");
    adb3_target_sim_write_reg32(reg_addr,32,reg_be(0 to 3),reg_data_wr(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Wrote XRM_GPIO_DA TRI " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_wr(0 to 3)) &
           " with enable 0b" & conv_string(reg_be(0 to 3)) &
           " to byte address " & conv_string(reg_addr);

    ds_comp.frontio_complete <= true;
    if (f_io_passed) then
      assert false report "Test Front IO completed: PASSED." severity note;
    else
      assert false report "Test Front IO completed: FAILED." severity error;
      ds_pass.frontio_passed <= false;
    end if;


    -----------------------------------------------
    -- PN4 GPIO Test (64-bit register interface) --
    -----------------------------------------------
    reg_be(0 to 3) := conv_byte_enable("1111");

    -- Write io data
    io_data_ms := PN4_DATA;
    io_data_ls := not(PN4_DATA);

    reg_addr := IO_ADDR_BASE + PN4_GPIO_P_DATA_REG_ADDR;
    reg_data_wr(0 to 3) := conv_byte_vector(io_data_ms);
    adb3_target_sim_write_reg32(reg_addr,33,reg_be(0 to 3),reg_data_wr(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Wrote PN4_GPIO_P DATA " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_wr(0 to 3)) &
           " with enable 0b" & conv_string(reg_be(0 to 3)) &
           " to byte address " & conv_string(reg_addr);

    reg_addr := IO_ADDR_BASE + PN4_GPIO_N_DATA_REG_ADDR;
    reg_data_wr(0 to 3) := conv_byte_vector(io_data_ls);
    adb3_target_sim_write_reg32(reg_addr,34,reg_be(0 to 3),reg_data_wr(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Wrote PN4_GPIO_N DATA " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_wr(0 to 3)) &
           " with enable 0b" & conv_string(reg_be(0 to 3)) &
           " to byte address " & conv_string(reg_addr);

    -- Write io enable
    reg_addr := IO_ADDR_BASE + PN4_GPIO_P_TRI_REG_ADDR;
    reg_data_wr(0 to 3) := conv_byte_vector(X"00000000");
    adb3_target_sim_write_reg32(reg_addr,35,reg_be(0 to 3),reg_data_wr(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Wrote PN4_GPIO_P TRI " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_wr(0 to 3)) &
           " with enable 0b" & conv_string(reg_be(0 to 3)) &
           " to byte address " & conv_string(reg_addr);

    reg_addr := IO_ADDR_BASE + PN4_GPIO_N_TRI_REG_ADDR;
    reg_data_wr(0 to 3) := conv_byte_vector(X"00000000");
    adb3_target_sim_write_reg32(reg_addr,36,reg_be(0 to 3),reg_data_wr(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Wrote PN4_GPIO_N TRI " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_wr(0 to 3)) &
           " with enable 0b" & conv_string(reg_be(0 to 3)) &
           " to byte address " & conv_string(reg_addr);

    -- Wait for data to get to outputs
    adb3_wait_cycles(50,ocp_clk);

    -- Read io data
    reg_addr := IO_ADDR_BASE + PN4_GPIO_P_DATA_REG_ADDR;
    adb3_target_sim_read_reg32(reg_addr,37,reg_data_rd(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Read PN4_GPIO_P DATA " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_rd(0 to 3)) &
           " from byte address " & conv_string(reg_addr);

    if (conv_vector(reg_data_rd(0 to 3))(PN4_GPIO_WIDTH-1 downto 0) /= io_data_ms(PN4_GPIO_WIDTH-1 downto 0)) then
      r_io_passed := false;
    end if;

    reg_addr := IO_ADDR_BASE + PN4_GPIO_N_DATA_REG_ADDR;
    adb3_target_sim_read_reg32(reg_addr,38,reg_data_rd(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Read PN4_GPIO_N DATA " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_rd(0 to 3)) &
           " from byte address " & conv_string(reg_addr);

    if (conv_vector(reg_data_rd(0 to 3))(PN4_GPIO_WIDTH-1 downto 0) /= io_data_ls(PN4_GPIO_WIDTH-1 downto 0)) then
      r_io_passed := false;
    end if;

    -- Write io tri-state disable
    reg_addr := IO_ADDR_BASE + PN4_GPIO_P_TRI_REG_ADDR;
    reg_data_wr(0 to 3) := conv_byte_vector(X"FFFFFFFF");
    adb3_target_sim_write_reg32(reg_addr,39,reg_be(0 to 3),reg_data_wr(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Wrote PN4_GPIO_P TRI " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_wr(0 to 3)) &
           " with enable 0b" & conv_string(reg_be(0 to 3)) &
           " to byte address " & conv_string(reg_addr);

    reg_addr := IO_ADDR_BASE + PN4_GPIO_N_TRI_REG_ADDR;
    reg_data_wr(0 to 3) := conv_byte_vector(X"FFFFFFFF");
    adb3_target_sim_write_reg32(reg_addr,40,reg_be(0 to 3),reg_data_wr(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Wrote PN4_GPIO_N TRI " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_wr(0 to 3)) &
           " with enable 0b" & conv_string(reg_be(0 to 3)) &
           " to byte address " & conv_string(reg_addr);


    -----------------------------------------------
    -- PN6 GPIO Test (32-bit register interface) --
    -----------------------------------------------
    reg_be(0 to 3) := conv_byte_enable("1111");

    -- Write io data
    io_data_ms := PN6_DATA(63 downto 32);
    io_data_ls := PN6_DATA(31 downto 0);

    reg_addr := IO_ADDR_BASE + PN6_GPIO_MS_DATA_REG_ADDR;
    reg_data_wr(0 to 3) := conv_byte_vector(io_data_ms);
    adb3_target_sim_write_reg32(reg_addr,41,reg_be(0 to 3),reg_data_wr(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Wrote PN6_GPIO_MS WDATA " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_wr(0 to 3)) &
           " with enable 0b" & conv_string(reg_be(0 to 3)) &
           " to byte address " & conv_string(reg_addr);

    reg_addr := IO_ADDR_BASE + PN6_GPIO_LS_DATA_REG_ADDR;
    reg_data_wr(0 to 3) := conv_byte_vector(io_data_ls);
    adb3_target_sim_write_reg32(reg_addr,42,reg_be(0 to 3),reg_data_wr(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Wrote PN6_GPIO_LS WDATA " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_wr(0 to 3)) &
           " with enable 0b" & conv_string(reg_be(0 to 3)) &
           " to byte address " & conv_string(reg_addr);

    -- Write io enable
    reg_addr := IO_ADDR_BASE + PN6_GPIO_MS_TRI_REG_ADDR;
    reg_data_wr(0 to 3) := conv_byte_vector(X"00000000");
    adb3_target_sim_write_reg32(reg_addr,43,reg_be(0 to 3),reg_data_wr(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Wrote PN6_GPIO_MS TRI " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_wr(0 to 3)) &
           " with enable 0b" & conv_string(reg_be(0 to 3)) &
           " to byte address " & conv_string(reg_addr);

    reg_addr := IO_ADDR_BASE + PN6_GPIO_LS_TRI_REG_ADDR;
    reg_data_wr(0 to 3) := conv_byte_vector(X"00000000");
    adb3_target_sim_write_reg32(reg_addr,44,reg_be(0 to 3),reg_data_wr(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Wrote PN6_GPIO_LS TRI " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_wr(0 to 3)) &
           " with enable 0b" & conv_string(reg_be(0 to 3)) &
           " to byte address " & conv_string(reg_addr);

    -- Wait for data to get to outputs
    adb3_wait_cycles(50,ocp_clk);

    -- Read io data
    reg_addr := IO_ADDR_BASE + PN6_GPIO_MS_DATA_REG_ADDR;
    adb3_target_sim_read_reg32(reg_addr,45,reg_data_rd(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Read PN6_GPIO_MS RDATA " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_rd(0 to 3)) &
           " from byte address " & conv_string(reg_addr);

    io_data_x2(63 downto 32) := conv_vector(reg_data_rd(0 to 3));

    reg_addr := IO_ADDR_BASE + PN6_GPIO_LS_DATA_REG_ADDR;
    adb3_target_sim_read_reg32(reg_addr,46,reg_data_rd(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Read PN6_GPIO_LS RDATA " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_rd(0 to 3)) &
           " from byte address " & conv_string(reg_addr);

    io_data_x2(31 downto 0) := conv_vector(reg_data_rd(0 to 3));

    if (io_data_x2(PN6_GPIO_WIDTH-1 downto 0) /= PN6_DATA(PN6_GPIO_WIDTH-1 downto 0)) then
      r_io_passed := false;
    end if;

    -- Write io tri-state disable
    reg_addr := IO_ADDR_BASE + PN6_GPIO_MS_TRI_REG_ADDR;
    reg_data_wr(0 to 3) := conv_byte_vector(X"FFFFFFFF");
    adb3_target_sim_write_reg32(reg_addr,47,reg_be(0 to 3),reg_data_wr(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Wrote PN6_GPIO_MS TRI " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_wr(0 to 3)) &
           " with enable 0b" & conv_string(reg_be(0 to 3)) &
           " to byte address " & conv_string(reg_addr);

    reg_addr := IO_ADDR_BASE + PN6_GPIO_LS_TRI_REG_ADDR;
    reg_data_wr(0 to 3) := conv_byte_vector(X"FFFFFFFF");
    adb3_target_sim_write_reg32(reg_addr,48,reg_be(0 to 3),reg_data_wr(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Wrote PN6_GPIO_LS TRI " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_wr(0 to 3)) &
           " with enable 0b" & conv_string(reg_be(0 to 3)) &
           " to byte address " & conv_string(reg_addr);

    ds_comp.reario_complete <= true;
    if (r_io_passed) then
      assert false report "Test Rear IO completed: PASSED." severity note;
    else
      assert false report "Test Rear IO completed: FAILED." severity error;
      ds_pass.reario_passed <= false;
    end if;


    --------------------
    -- Interrupt Test --
    --------------------
    reg_be(0 to 3) := conv_byte_enable("1111");

    -- Read OOR
    reg_addr := BRAM_ADDR_BASE + BRAM_ADDR_MASK + 1;
    adb3_target_sim_read(reg_addr,12,reg_data_rd(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Read OOR Addr 0 " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_rd(0 to 3)) &
           " from byte address " & conv_string(reg_addr);

    if (reg_data_rd(0 to 3) /= conv_byte_vector(X"DEADC0DE")) then
      int_passed := false;
    end if;

    -- Write interrupt MASK
    reg_addr := INT_ADDR_BASE + INT_WMASK_REG_ADDR;
    reg_data_wr(0 to 3) := conv_byte_vector(MASK_EN_ALL);
    adb3_target_sim_write_reg32(reg_addr,49,reg_be(0 to 3),reg_data_wr(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Wrote Interrupt MASK " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_wr(0 to 3)) &
           " with enable 0b" & conv_string(reg_be(0 to 3)) &
           " to byte address " & conv_string(reg_addr);

    -- Read interrupt MASK
    reg_addr := INT_ADDR_BASE + INT_RMASK_REG_ADDR;
    adb3_target_sim_read_reg32(reg_addr,50,reg_data_rd(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Read Interrupt MASK " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_rd(0 to 3)) &
           " from byte address " & conv_string(reg_addr);

    if (reg_data_rd(0 to 3) /= conv_byte_vector(MASK_EN_ALL)) then
      int_passed := false;
    end if;

    -- Write interrupt COUNT
    reg_addr := INT_ADDR_BASE + INT_WCOUNT_REG_ADDR;
    reg_data_wr(0 to 3) := conv_byte_vector(X"FFFFFFFF");
    adb3_target_sim_write_reg32(reg_addr,51,reg_be(0 to 3),reg_data_wr(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Wrote Interrupt COUNT " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_wr(0 to 3)) &
           " with enable 0b" & conv_string(reg_be(0 to 3)) &
           " to byte address " & conv_string(reg_addr);

    -- Read interrupt COUNT
    reg_addr := INT_ADDR_BASE + INT_RCOUNT_REG_ADDR;
    adb3_target_sim_read_reg32(reg_addr,52,reg_data_rd(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Read Interrupt COUNT " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_rd(0 to 3)) &
           " from byte address " & conv_string(reg_addr);

    if (reg_data_rd(0 to 3) /= conv_byte_vector(X"FFFFFFFF")) then
      int_passed := false;
    end if;

    -- Cycle through set-detect-clear for all 32 interrupts
    for i in 0 to 31 loop
      int_set := (others => '0');
      int_set(i) := '1';

      -- Write interrupt SET
      reg_addr := INT_ADDR_BASE + INT_SET_REG_ADDR;
      reg_data_wr(0 to 3) := conv_byte_vector(int_set);
      adb3_target_sim_write_reg32(reg_addr,53,reg_be(0 to 3),reg_data_wr(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
--      report "Wrote Interrupt SET " & integer'image(reg_xfer) &
--             " bytes 0x" & conv_string_hex(reg_data_wr(0 to 3)) &
--             " with enable 0b" & conv_string(reg_be(0 to 3)) &
--             " to byte address " & conv_string(reg_addr);

      loop
        exit when (linti_n_edge = '1');   -- Wait for next edge on linti_n (Shared variable defined in uber_tb_pkg)
        adb3_wait_cycles(1,ocp_clk);
      end loop;

      -- Interrupt response time of 15 cycles
      -- Unreastically short for purposes of simulation
      adb3_wait_cycles(15,ocp_clk);

      -- Check interrupt status register
      reg_addr := INT_ADDR_BASE + INT_STAT_REG_ADDR;
      adb3_target_sim_read_reg32(reg_addr,54,reg_data_rd(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
--      report "Read Interrupt STAT " & integer'image(reg_xfer) &
--             " bytes 0x" & conv_string_hex(reg_data_rd(0 to 3)) &
--             " from byte address " & conv_string(reg_addr);
      int_stat := conv_vector(reg_data_rd(0 to 3));

      -- Mask the STAT register value just read with the value of the MASK register
      int_stat := int_stat and not(MASK_EN_ALL);

      -- Clear the unmasked interrupts
      reg_addr := INT_ADDR_BASE + INT_CLEAR_REG_ADDR;
      reg_data_wr(0 to 3) := conv_byte_vector(int_stat);
      adb3_target_sim_write_reg32(reg_addr,55,reg_be(0 to 3),reg_data_wr(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
      report "Interrupt Handler: Cleared interrupt(s), masked STAT = 0x" & conv_string_hex(reg_data_wr(0 to 3));

      -- Rearm the interrupt logic
      linti_n_edge        := '0';
      reg_addr := INT_ADDR_BASE + INT_ARM_REG_ADDR;
      reg_data_wr(0 to 3) := conv_byte_vector(X"deadbeef"); -- actual value does not matter for ARM register
      adb3_target_sim_write_reg32(reg_addr,56,reg_be(0 to 3),reg_data_wr(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    end loop;

    -- Read interrupt STAT data
    reg_addr := INT_ADDR_BASE + INT_STAT_REG_ADDR;
    adb3_target_sim_read_reg32(reg_addr,57,reg_data_rd(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Read Interrupt STAT " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_rd(0 to 3)) &
           " from byte address " & conv_string(reg_addr);

    if (reg_data_rd(0 to 3) /= conv_byte_vector(X"00000000")) then
      int_passed := false;
    end if;

    ds_comp.int_complete <= true;
    if (int_passed) then
      assert false report "Test Interrupt completed: PASSED." severity note;
    else
      assert false report "Test Interrupt completed: FAILED." severity error;
      ds_pass.int_passed <= false;
    end if;


    -------------------------------------------
    -- Info Test (32-bit register interface) --
    -------------------------------------------
    reg_be(0 to 3) := conv_byte_enable("1111");

    -- Read OOR
    reg_addr := BRAM_ADDR_BASE + BRAM_ADDR_MASK + 1;
    adb3_target_sim_read(reg_addr,12,reg_data_rd(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Read OOR Addr 0 " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_rd(0 to 3)) &
           " from byte address " & conv_string(reg_addr);

    if (reg_data_rd(0 to 3) /= conv_byte_vector(X"DEADC0DE")) then
      info_passed := false;
    end if;

    -- Read DATE from register
    reg_addr := INFO_ADDR_BASE + INFO_DATE_REG_ADDR;
    adb3_target_sim_read_reg32(reg_addr,58,reg_data_rd(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Read Info DATE " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_rd(0 to 3)) &
           " from byte address " & conv_string(reg_addr);

    if (conv_vector(reg_data_rd(0 to 3)) /= TODAYS_DATE) then
      info_passed := false;
    end if;

    -- Read DATE from register
    reg_addr := INFO_ADDR_BASE + INFO_TIME_REG_ADDR;
    adb3_target_sim_read_reg32(reg_addr,59,reg_data_rd(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Read Info TIME " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_rd(0 to 3)) &
           " from byte address " & conv_string(reg_addr);

    if (conv_vector(reg_data_rd(0 to 3)) /= TODAYS_TIME) then
      info_passed := false;
    end if;

    -- Read BRAM addr base from register
    reg_addr := INFO_ADDR_BASE + INFO_BASE_REG_ADDR;
    adb3_target_sim_read_reg32(reg_addr,60,reg_data_rd(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Read Info BASE " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_rd(0 to 3)) &
           " from byte address " & conv_string(reg_addr);

    if (conv_vector(reg_data_rd(0 to 3))(DS_ADDR_WIDTH-1 downto 0) /= BRAM_ADDR_BASE) then
      info_passed := false;
    end if;

    -- Read BRAM addr mask from register
    reg_addr := INFO_ADDR_BASE + INFO_MASK_REG_ADDR;
    adb3_target_sim_read_reg32(reg_addr,61,reg_data_rd(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Read Info MASK " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_rd(0 to 3)) &
           " from byte address " & conv_string(reg_addr);

    if (conv_vector(reg_data_rd(0 to 3))(DS_ADDR_WIDTH-1 downto 0) /= BRAM_ADDR_MASK) then
      info_passed := false;
    end if;

    ds_comp.info_complete <= true;
    if (info_passed) then
      assert false report "Test Info completed: PASSED." severity note;
    else
      assert false report "Test Info completed: FAILED." severity error;
      ds_pass.info_passed <= false;
    end if;


    -----------------------------------
    -- BRAM Test (128-bit interface) --
    -----------------------------------
    reg_be( 0 to 15) := conv_byte_enable("11111111" & "11111111");
    reg_be(16 to 31) := conv_byte_enable("11111111" & "11111111");

    -- Test lower address limit
    -- Write BRAM
    reg_addr := BRAM_ADDR_BASE;
    reg_data_wr(0 to 3) := conv_byte_vector(X"2389EF45");
    adb3_target_sim_write_reg32(reg_addr,66,reg_be(0 to 3),reg_data_wr(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Wrote BRAM Addr base " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_wr(0 to 3)) &
           " with enable 0b" & conv_string(reg_be(0 to 3)) &
           " to byte address " & conv_string(reg_addr);

    -- Read BRAM
    reg_addr := BRAM_ADDR_BASE;
    adb3_target_sim_read_reg32(reg_addr,67,reg_data_rd(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Read BRAM Addr base " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_rd(0 to 3)) &
           " from byte address " & conv_string(reg_addr);

    if (reg_data_rd(0 to 3) /= conv_byte_vector(X"2389EF45")) then
      bram_passed := false;
    end if;

    -- Test burst length = 1, 1 x 16-byte transfer
    -- Write BRAM
    reg_addr := BRAM_ADDR_BASE;
    reg_data_wr(0 to 15) := conv_byte_vector(X"56789ABC" & X"DEF12345" & X"6789ABCD" & X"EF123456");
    adb3_target_sim_write(reg_addr,62,reg_be(0 to 15),reg_data_wr(0 to 15),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Wrote BRAM Addr base " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_wr(0 to 15)) &
           " with enable 0b" & conv_string(reg_be(0 to 15)) &
           " to byte address " & conv_string(reg_addr);

    -- Read BRAM
    reg_addr := BRAM_ADDR_BASE;
    adb3_target_sim_read(reg_addr,63,reg_data_rd(0 to 15),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Read BRAM Addr base " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_rd(0 to 15)) &
           " from byte address " & conv_string(reg_addr);

    if (reg_data_rd(0 to 15) /= conv_byte_vector(X"56789ABC" & X"DEF12345" & X"6789ABCD" & X"EF123456")) then
      bram_passed := false;
    end if;

    -- Test burst length > 1, 2 x 16-byte transfer
    -- Write BRAM
    reg_addr := BRAM_ADDR_BASE;
    reg_data_wr( 0 to 15) := conv_byte_vector(X"12345678" & X"9ABCDEF1" & X"23456789" & X"ABCDEF12");
    reg_data_wr(16 to 31) := conv_byte_vector(X"FEDCBA98" & X"7654321F" & X"EDCBA987" & X"654321FE");
    adb3_target_sim_write(reg_addr,64,reg_be(0 to 31),reg_data_wr(0 to 31),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Wrote BRAM Addr base " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_wr(0 to 31)) &
           " with enable 0b" & conv_string(reg_be(0 to 31)) &
           " to byte address " & conv_string(reg_addr);

    -- Read BRAM
    reg_addr := BRAM_ADDR_BASE;
    adb3_target_sim_read(reg_addr,65,reg_data_rd(0 to 31),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Read BRAM Addr base " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_rd(0 to 31)) &
           " from byte address " & conv_string(reg_addr);

    if (reg_data_rd( 0 to 15) /= conv_byte_vector(X"12345678" & X"9ABCDEF1" & X"23456789" & X"ABCDEF12"))or
       (reg_data_rd(16 to 31) /= conv_byte_vector(X"FEDCBA98" & X"7654321F" & X"EDCBA987" & X"654321FE")) then
      bram_passed := false;
    end if;

    -- Test beyond lower address limit (if > 0)
    -- Write BRAM
    if not(OR_reduce(BRAM_ADDR_BASE) = '0') then
      reg_addr := BRAM_ADDR_BASE - 4;
      reg_data_wr(0 to 3) := conv_byte_vector(X"369CF258");
      adb3_target_sim_write_reg32(reg_addr,68,reg_be(0 to 3),reg_data_wr(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
      report "Wrote OOR Addr base-4 " & integer'image(reg_xfer) &
             " bytes 0x" & conv_string_hex(reg_data_wr(0 to 3)) &
             " with enable 0b" & conv_string(reg_be(0 to 3)) &
             " to byte address " & conv_string(reg_addr);

      -- Read BRAM
      reg_addr := BRAM_ADDR_BASE - 4;
      adb3_target_sim_read_reg32(reg_addr,69,reg_data_rd(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
      report "Read OOR Addr base-4 " & integer'image(reg_xfer) &
             " bytes 0x" & conv_string_hex(reg_data_rd(0 to 3)) &
             " from byte address " & conv_string(reg_addr);

      if (reg_data_rd(0 to 3) /= conv_byte_vector(X"DEADC0DE")) then
        bram_passed := false;
      end if;
    end if;

    -- Test beyond upper address limit
    -- Write OOR
    reg_addr := BRAM_ADDR_BASE + BRAM_ADDR_MASK + 1;
    reg_data_wr(0 to 3) := conv_byte_vector(X"258BE147");
    adb3_target_sim_write_reg32(reg_addr,70,reg_be(0 to 3),reg_data_wr(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Wrote OOR Addr top+1 " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_wr(0 to 3)) &
           " with enable 0b" & conv_string(reg_be(0 to 3)) &
           " to byte address " & conv_string(reg_addr);

    -- Read OOR
    reg_addr := BRAM_ADDR_BASE + BRAM_ADDR_MASK + 1;
    adb3_target_sim_read_reg32(reg_addr,71,reg_data_rd(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Read OOR Addr top+1 " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_rd(0 to 3)) &
           " from byte address " & conv_string(reg_addr);

    if (reg_data_rd(0 to 3) /= conv_byte_vector(X"DEADC0DE")) then
      bram_passed := false;
    end if;

    -- Test burst length > 1, 2 x 16-byte transfer
    -- Write OOR
    reg_addr := BRAM_ADDR_BASE + BRAM_ADDR_MASK + 1;
    reg_data_wr( 0 to 15) := conv_byte_vector(X"12345678" & X"9ABCDEF1" & X"23456789" & X"ABCDEF12");
    reg_data_wr(16 to 31) := conv_byte_vector(X"FEDCBA98" & X"7654321F" & X"EDCBA987" & X"654321FE");
    adb3_target_sim_write(reg_addr,64,reg_be(0 to 31),reg_data_wr(0 to 31),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Wrote OOR Addr top+1 " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_wr(0 to 31)) &
           " with enable 0b" & conv_string(reg_be(0 to 31)) &
           " to byte address " & conv_string(reg_addr);

    -- Read OOR
    reg_addr := BRAM_ADDR_BASE + BRAM_ADDR_MASK + 1;
    adb3_target_sim_read(reg_addr,65,reg_data_rd(0 to 31),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Read OOR Addr top+1 " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_rd(0 to 31)) &
           " from byte address " & conv_string(reg_addr);

    if (reg_data_rd( 0 to 15) /= conv_byte_vector(X"DEADC0DE" & X"DEADC0DE" & X"DEADC0DE" & X"DEADC0DE"))or
       (reg_data_rd(16 to 31) /= conv_byte_vector(X"DEADC0DE" & X"DEADC0DE" & X"DEADC0DE" & X"DEADC0DE")) then
      bram_passed := false;
    end if;

    -- Test upper address limit
    -- Write BRAM
    reg_addr := BRAM_ADDR_BASE + BRAM_ADDR_MASK - 3;
    reg_data_wr(0 to 3) := conv_byte_vector(X"147AD036");
    adb3_target_sim_write_reg32(reg_addr,66,reg_be(0 to 3),reg_data_wr(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Wrote BRAM Addr top " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_wr(0 to 3)) &
           " with enable 0b" & conv_string(reg_be(0 to 3)) &
           " to byte address " & conv_string(reg_addr);

    -- Read BRAM
    reg_addr := BRAM_ADDR_BASE + BRAM_ADDR_MASK - 3;
    adb3_target_sim_read_reg32(reg_addr,67,reg_data_rd(0 to 3),reg_xfer,ocp_clk,ds_m2s,ds_s2m);
    report "Read BRAM Addr top " & integer'image(reg_xfer) &
           " bytes 0x" & conv_string_hex(reg_data_rd(0 to 3)) &
           " from byte address " & conv_string(reg_addr);

    if (reg_data_rd(0 to 3) /= conv_byte_vector(X"147AD036")) then
      bram_passed := false;
    end if;


    ds_comp.bram_complete <= true;
    if (bram_passed) then
      assert false report "Test BRAM completed: PASSED." severity note;
    else
      assert false report "Test BRAM completed: FAILED." severity error;
      ds_pass.bram_passed <= false;
    end if;

    wait;
  end process;

  -- Connect direct slave output
  direct_slave_m2s <= ds_m2s;

end test;